%   Transport through magnetic barrier -  Based on Favago TPI v4.4
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.

%%
function magnetic_barrier( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

toCalculate = 1;
Continue    = 0;
outfilename = ['Ribbon_Conductivity',num2str( filenum ),'.mat'];
outputdir   = [];
setOutputDir();

height = 300;
width = 160;


Emin = 0.001; %In EV
Emax = 0.6;
Enum = 200;
Evec = Emin:(Emax-Emin)/(Enum+1):Emax;


B = 40e-0; %in Tesla
homegac = [];


% calculated conductance (and number of open channels) as a function of B
Conductance = zeros(1,length(Evec)); 
nyitott_csatornak = zeros(1,length(Evec));
DeltaC      = zeros(1,length(Evec));

ribbon = [];
WS_ribbon = [];

if Continue && toCalculate
  HandleForLoad = LoadFromFile( [outputdir,'/',outfilename] );
  Conductance = HandleForLoad.LoadVariable('Conductance', 'NoEmpty', 'Off');
  DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'Off');
  nyitott_csatornak = HandleForLoad.LoadVariable('nyitott_csatornak', 'NoEmpty', 'On');
  homegac = HandleForLoad.LoadVariable('homegac', 'NoEmpty', 'Off');
  Evec = HandleForLoad.LoadVariable('Evec', 'NoEmpty', 'On');
  HandleForLoad.ClearLoadedVariables(); 
end


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
    
        if ~toCalculate 
             disp('Loading Calculated data')
            HandleForLoad = LoadFromFile( [outputdir,'/',outfilename] );
            Conductance = HandleForLoad.LoadVariable('Conductance', 'NoEmpty', 'On');
            DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'On');
            nyitott_csatornak = HandleForLoad.LoadVariable('nyitott_csatornak', 'NoEmpty', 'On');
            homegac = HandleForLoad.LoadVariable('homegac', 'NoEmpty', 'Off');
            Evec = HandleForLoad.LoadVariable('Widthvec', 'NoEmpty', 'On');
            HandleForLoad.ClearLoadedVariable();            
            setOutputDir();
            return
        end
        
        setOutputDir();
        
        
         if Continue && toCalculate
           indexes2calc = find( Conductance == 0 );
        else
           indexes2calc = 1:length(Evec);
        end
        
        for idx = 1:length(indexes2calc)  
            Energy = Evec(indexes2calc(idx));     
            ribbon = Ribbon('width', width, 'height', height );
            WS_ribbon = ribbon.GetWorkspace();
            
            rCC = WS_ribbon.rCC*1e-10; %Angstrom
            phi0 = WS_ribbon.h/WS_ribbon.qe; % magnetic flux quantum
            eta_B = 2*pi/phi0*(rCC)^2*B;
            if B>1e-8
                homegac = 2.97*sqrt(9/2*eta_B);
            end

       
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
        
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductance for the given magentic field')  

            
            tic
            [Conductance_tmp,nyitott_csatornak_num,DeltaC_tmp] = Transport( Energy, B );
            toc
            
            Conductance(indexes2calc(idx)) = Conductance_tmp;
            nyitott_csatornak(indexes2calc(idx)) = nyitott_csatornak_num(1);
            DeltaC(indexes2calc(idx)) = DeltaC_tmp;
            
            
            
            save( [outputdir,'/',outfilename], 'Conductance', 'nyitott_csatornak', 'Evec', 'DeltaC', 'height', 'width', 'homegac' );
            disp([ num2str(indexes2calc(idx)/length(Evec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
            
            
        end
                
    end
    
%% Calculates the conductance
    function [Conductance,nyitott_csatornak,DeltaC] = Transport( Energy, B )
    
    	CreateHandlesForMagneticField( B )    	
    	ribbon.setEnergy( Energy )
        
        ribbon.CalcFiniteGreensFunction( 'gauge_trans', true );        
        
        
        Dysonfunc = @()ribbon.CustomDysonFunc( 'constant_channels', true, 'SelfEnergy', true );
        try
            WS_ribbon.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        catch errCause
            err = MException('magnetic_barrier:Transport', 'Error occured in calculating the Greens function');
            err = addCause(err, errCause);
            save('Error_magnetic_barrier_Transport.mat');
            Conductance = NaN;
            nyitott_csatornak = NaN(2,1);
            DeltaC = NaN;
            return
        end
        [S,nyitott_csatornak] = WS_ribbon.FL_handles.SmatrixCalc();
        
        norma = norm(S*S'-eye(sum(nyitott_csatornak)));
        
        if norma >= 1e-3 || isnan(norma)
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if nyitott_csatornak(1) ~= nyitott_csatornak(2)
           disp( 'openchannels do not match' )
           nyitott_csatornak
        end
        
        try
        	conductance = WS_ribbon.FL_handles.Conduktance();
        catch
            Conductance = NaN;
            DeltaC = NaN;
            return
        end
        
        	conductance = abs([conductance(1,:), conductance(2,:)]);
        	DeltaC = std(conductance);
        
        	C = mean(conductance);
        	Conductance = C;
        
        	disp( ['E = ', num2str(Energy), ' conductance = ', num2str(C)])   
    
    end
    
    
%% this creates handles to apply magnetic filed in the ribbon Hamiltonians 
    function CreateHandlesForMagneticField( B )        
        [hLandaux, hLandauy, gauge_field ] = createVectorPotential( B );        
        ribbon.setHandlesForMagneticField('scatter', hLandauy, 'lead', hLandaux, 'gauge_field', gauge_field );          
    end
    

%% creates handle for vector potential
    function [hLandaux, hLandauy, hgauge_field ] = createVectorPotential( B )
    
    	rCC = WS_ribbon.rCC*1e-10; %Angstrom
		phi0 = WS_ribbon.h/WS_ribbon.qe;
		
        eta_B = 2*pi/phi0*(rCC)^2*B;
        Aconst = 0.1; % constant vektor potential in the systems: stabilizes the numerical computation
        
        Opt = WS_ribbon.FL_handles.Read( 'Opt' );
        
        if strcmp(Opt.Lattice_Type, 'Square')
            lattice_constant = 1;
        else
            lattice_constant = sqrt(3);
        end
                
        hLandaux = @Landaux;
        %hLandauy = @(x,y)([zeros( 1,length(x) );0.5*ones( 1,length(x) )]);
        hLandauy = @(x,y)([0;eta_B*x]);
        hgauge_field = @gauge_field;
        
        %---------------------------------
        % gauge scalar fiald for transformation From Landauy to Landaux
        function ret = gauge_field(x,y)
        	%ret = -0.5*y - 0.5*x;
            %return
            ret = zeros( length(x),1 );
            
            indexes1 = logical( y<=0 );
            ret( indexes1 ) = -Aconst*x(indexes1);
            
            indexes2 = logical( y>=height*lattice_constant );
            ret( indexes2 ) = -eta_B*height*lattice_constant*x(indexes2) - Aconst*x(indexes2);
            
            indexes3 = ~( indexes1 | indexes2 );
            ret( indexes3 ) = -eta_B*x(indexes3).*y(indexes3) - Aconst*x(indexes3);  
        end
              
        
        %-----------------------------------
        function ret = Landaux( x,y )
    	    %ret = -[0.5*ones( 1,length(x) ); zeros( 1,length(x) )];
	        %return
            ret = zeros( length(x),2 );
            
            indexes1 = logical( y<=0 );
            ret( indexes1, 1 ) = -Aconst;
            
            indexes2 = logical( y>=height*lattice_constant );
            ret( indexes2, 1 ) = -eta_B*height*lattice_constant-Aconst;
            
            indexes3 = ~( indexes1 | indexes2 );
            ret( indexes3, 1) = -eta_B*y(indexes3)-Aconst;
        end
        
    end



%% sets output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = [ resultsdir ];        
    end




    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductance
               
        
        indexek = logical( (abs(DeltaC) <= 0.5) & (Evec~=0) & ~isnan(DeltaC) );
        x_lim = [min(Evec(indexek)) max(Evec(indexek))];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(Evec(indexek), Conductance(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);  
        try
            plot(Evec(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
        
        if ~isempty(homegac)
            nmax = round( (Emax/homegac)^2 );
            nvec = 1:nmax;
            for idx = 1:length(nvec)
                plot(sqrt([nvec(idx), nvec(idx)])*homegac, [0 max(Conductance(indexek))], 'Linewidth', 0.5, 'color', [0 0 1], 'Parent', axes_cond);
            end
        end
        
               
        
        
        %plot(Evec(indexek), nyitott_csatornak(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);    
        
         % Create xlabel
        %xlabel_position = [0 0 0];
        xlabel('E [ev]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %xlabel_handle = get(axes_cond,'XLabel');  
        %set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        %ylabel_position = [0 0 0];
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %ylabel_handle = get(axes_cond,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        
        %set(get(get(numerics,'Annotation'),'LegendInformation'), 'IconDisplayStyle','off'); % Exclude line from legend

        fig_legend = legend(axes_cond, {'conductance', 'unitarity'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', [outputdir,'/Ribbon_Conductivity_', num2str(filenum),'.eps'])
        print('-dpdf', [outputdir,'/Ribbon_Conductivity_', num2str(filenum),'.pdf'])
        if toCalculate
            close(figure1)
        end
        
        
    end




end
