%%    Favago Transport Programming Interface - CombineRibbons
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for combining multiple ribbons of equal width from the same material.
%%
function handles = CombineRibbons( varargin )

WS = ws_CombineRibbons;

ribbon    = [];
WS_ribbon = [];

gfin        = [];
gfininv     = [];
coordinates = structures('coordinates');

CommonFunctionsHandle = CommonFunctions();

InputParsing();


handles.GetWorkspace                = @GetWorkspace;
handles.create_combined_GreensFunction = @create_combined_GreensFunction;
handles.addRibbon                  = @addRibbon;
handles.Transport                  = @Transport;
handles.getGfininv                 = @getGfininv;
handles.getTotalHeight             = @getTotalHeight;
handles.getHeights                 = @getHeights;
handles.setHandlesForMagneticField = @setHandlesForMagneticField;



%% Transport
% Calculate transport --- use for development pupose only in "contact/scattering center/contact" arrangement.
    function [Conductance,ny,DeltaC] = Transport( Energy )
        setEnergy( Energy )
        
        create_combined_GreensFunction('onlygfininverz', true);
        
        Surface_sc = cell(2,1);
        WS_ribbon_tmp = WS.Ribbons{1}.GetWorkspace();
        Surface_sc{1} = WS_ribbon_tmp.Surface_tmp.CreateClone();
        Surface_sc{1}.ShiftCoordinates( -1 )
        WS_ribbon_tmp = WS.Ribbons{end}.GetWorkspace();
        total_height = getTotalHeight();
        Surface_sc{2} = WS_ribbon_tmp.Surface_tmp.CreateClone();
        Surface_sc{2}.ShiftCoordinates( total_height )
        
        
        Dysonfunc = @()ribbon.CustomDysonFunc( 'constant_channels', true, 'gfininv', gfininv, 'Surface_sc', Surface_sc);
        try
            WS_ribbon.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        catch errCause
            err = MException('COmbineRibbons:Transport', 'Error occured in calculating the Greens function');
            err = addCause(err, errCause);
            save('Error_CombineRibbons_Transport.mat');
            Conductance = NaN;
            ny = NaN(2,1);
            DeltaC = NaN;
            throw(err);
        end
        [S,ny] = WS_ribbon.FL_handles.SmatrixCalc();
        
        norma = norm(S*S'-eye(sum(ny)));
            
        if norma >= 1e-3
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if ny(1) ~= ny(2)
           disp( ['openchannels do not match: ',num2str(ny)] )
        end
        
        
        conductance = WS_ribbon.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(2,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        Conductance = C;
        
        disp( ['conductance = ', num2str(C), ' open_channels= ', num2str(ny(1)), ' DeltaC = ', num2str(DeltaC)])        
        %2/pi
        
    end


%% create_combined_GreensFunction
% Calculates the surface Greens function of the combined ribbons by a recursive method.
function create_combined_GreensFunction( varargin )
    p = inputParser;
    p.addParamValue('E', [], @isscalar); %The energy to be used in the ribbons
    p.addParamValue('onlygfininverz', false );
	p.parse(varargin{:});

    E            = p.Results.E;
    onlygfininverz    = p.Results.onlygfininverz;
    
    gfininv = [];
	
    for idx = 1:length( WS.Ribbons )
		ribbon_tmp = WS.Ribbons{idx};
        if ~isempty( E )
			ribbon_tmp.setEnergy( E );
        end
    end
    
    % creating the interface regions if needed
    if ~isempty( WS_ribbon.Surface_interface )
       createInterfaceRegions()        
    end
    
    for idx = 1:length(WS.Ribbons)
       attachRibbon( WS.Ribbons{idx}, 'onlygfininverz', onlygfininverz );
    end
    
    % gauge transformation on the calculated Greens function
    if ~isempty( WS_ribbon.PeierlsTransform_Scatter ) 
        if ~isempty( gfin )        
            gfin = WS_ribbon.PeierlsTransform_Scatter.gaugeTransformation( gfin, coordinates, WS_ribbon.gauge_field );
        else
            gfininv = WS_ribbon.PeierlsTransform_Scatter.gaugeTransformation( gfininv, coordinates, WS_ribbon.gauge_field );
        end           
    end
        
end

%% getGfininv
% Reads out the inverse greens function.
function ret = getGfininv()
	ret = gfininv;
end


%% addRibbon
% Adds a Ribbon interface to the ribbon structure.
function addRibbon( ribbon2add, varargin )
    
    p = inputParser;
    p.addParamValue('shiftcoordinates', true );        
    p.parse(varargin{:});       
    shiftcoordinates    = p.Results.shiftcoordinates;
    
    if ~isempty( WS_ribbon.E )
		ribbon2add.setEnergy( WS_ribbon.E );
    end
    
    % checking if the width of the ribbons is equal
    if checkwidth( ribbon2add ) == -1
        err = MException('CombineRibbons:ribbon2add', 'widths of the ribbons are not identical.');
        save('Error_CombineRibbons_ribbon2add.mat');
        throw(err);
    end
    
    
    
    % shift the coordinates
    if shiftcoordinates
        total_height = getTotalHeight();
        if ~isempty(WS.Ribbons)
        	total_height = total_height + 1; % 1 for the coupling length
        	WS_ribbon2add = ribbon2add.GetWorkspace();
        	if ~isempty(WS_ribbon2add.Surface_interface)
            	total_height = total_height + 1;
        	end
        end
        ribbon2add.ShiftCoordinates( total_height );
    end
    
    
    WS.Ribbons{end+1} = ribbon2add;    
   
    WS_ribbon.height = getTotalHeight();
    
end



%% getTotalHeight
% Gets the total height of the ribbon structure.
    function ret = getTotalHeight()
        heights = getHeights();
        ret = sum( heights );
        
        if length(heights) == 1
            return
        end
        
        % accounting for the coupling between the ribbons
        if ~isempty( heights )
        	ret = ret + length(WS.Ribbons )-1;
       	end
        
        % accounting for the interface regions
        for idx = 2:length( WS.Ribbons )
            WS_ribbon_tmp = WS.Ribbons{idx}.GetWorkspace();
            if ~isempty(WS_ribbon_tmp.Surface_interface)
                ret = ret + 1;
            end
        end
    end

% getHeights
% Returns the list of the heights of the ribbon parts.
    function ret = getHeights()
        ribbonnum = length( WS.Ribbons );
        ret = zeros( ribbonnum, 1); 
        for idx = 1:ribbonnum
            WS_ribbon_tmp = WS.Ribbons{idx}.GetWorkspace();
            ret(idx) =  WS_ribbon_tmp.height;
        end
    end
    

%% setHandlesForMagneticField
% Sets the function handles of the vector potential and gauge transformation.
    function setHandlesForMagneticField( varargin )        
    
        p = inputParser;
        p.addParamValue('scatter', @(x,y)([0,0]) );
        p.addParamValue('lead', @(x,y)([0,0]) );
        p.addParamValue('gauge_field', [] ); 
        
        p.parse(varargin{:});

       
        hscatter    = p.Results.scatter;
        hlead       = p.Results.lead;
        gauge_field = p.Results.gauge_field;
               
        ribbon.setHandlesForMagneticField('scatter', hscatter, 'lead', hlead, 'gauge_field', gauge_field );    
        for idx = 1:length(WS.Ribbons)
        	WS.Ribbons{idx}.setHandlesForMagneticField('scatter', hscatter, 'lead', hscatter);%hlead, 'gauge_field', gauge_field );         
        end      
    end
    
    

%% GetWorkspace()
% Return the handle class to the workspace of the given interface. Changes in fields of the workspace are not always safe, since reinitialization of function handles are not performed.
    function ret = GetWorkspace()
        ret = WS;        
    end    
    
    

% attach the next ribbon to the ribbon sequence, and calculate the surface Greens function. 
function attachRibbon( ribbon_tmp, varargin )
    
        p = inputParser;
        p.addParamValue('onlygfininverz', false );
        p.addParamValue('decimate', true );
        
        p.parse(varargin{:});
       
        onlygfininverz    = p.Results.onlygfininverz;
        decimate           = p.Results.decimate;
        
            
    if isempty(gfininv) && isempty(gfin)
        ribbon_tmp.CalcFiniteGreensFunction( 'onlyGinv', true, 'gauge_trans', false );
        [~,gfininv] = ribbon_tmp.GetFiniteGreensFunction();
        coordinates = ribbon_tmp.getCoordinates();
    else
        if isempty(gfininv)
            if isnan(rcond(gfin) ) || abs( rcond(gfin) ) < 1e-15
                gfininv = CommonFunctionsHandle.inv_SVD( G );
            else
                gfininv = inv(gfin);
            end       
            gfin = [];
        end
        WS_ribbon_tmp = ribbon_tmp.GetWorkspace();
        size_gfin = size(gfininv);
        ribbon_tmp.CalcFiniteGreensFunction( 'onlyGinv', true, 'gauge_trans', false );
        [~,gfininv_tmp] = ribbon_tmp.GetFiniteGreensFunction();
        size_gfin_tmp = size( gfininv_tmp);
            
        Surface_sc = WS_ribbon_tmp.Surface_tmp.CreateClone();
            
        if isempty( WS_ribbon_tmp.Surface_interface )
                
            H1_sc_tmp    = Surface_sc.Read( 'H1' );
            H1adj_sc_tmp = Surface_sc.Read( 'H1adj' );
            size_H1_sc_tmp = size(H1_sc_tmp);
                
            H1_sc = zeros(size_gfin(1), size_gfin_tmp(2) );
            H1adj_sc = zeros(size_gfin_tmp(1), size_gfin(2) );
                
            H1_sc(end-size_H1_sc_tmp(1)+1:end, 1:size_H1_sc_tmp(2)) = H1_sc_tmp;
            H1_sc_tmp = [];
                
            H1adj_sc( 1:size_H1_sc_tmp(2), end-size_H1_sc_tmp(1)+1:end) = H1adj_sc_tmp;
            H1adj_sc_tmp = [];
                
            gfininv = [gfininv, -H1_sc; -H1adj_sc, gfininv_tmp];
            
            coordinates_tmp = ribbon_tmp.getCoordinates();      
            fnames = fieldnames(coordinates);
            for idx = 1:length( fnames )
                fname = fnames{idx};
                if strcmp( fname, 'a' )
                    continue
                else
                    coordinates.(fname) = [coordinates.(fname); coordinates_tmp.(fname)];
                end
            end
            
                
                
        else
                
            H0_interface = WS_ribbon_tmp.Surface_interface{1}.Read('H0');
            H1_interface_tmp = WS_ribbon_tmp.Surface_interface{1}.Read('H1');
            H1adj_interface_tmp = WS_ribbon_tmp.Surface_interface{1}.Read('H1adj');
            size_interface = size(H0_interface,1);
            size_H1_interface_tmp = size( H1_interface_tmp );
                
            % ribbon_tmp -> interface
            H1_sc_tmp    = Surface_sc.Read( 'H1' );
            H1adj_sc_tmp = Surface_sc.Read( 'H1adj' );
            size_H1_sc_tmp = size(H1_sc_tmp);
                
            H1_sc = zeros(size_interface, size_gfin_tmp(2) );
            H1adj_sc = zeros(size_gfin_tmp(1), size_interface );
                
            H1_sc(end-size_H1_sc_tmp(1)+1:end, 1:size_H1_sc_tmp(2)) = H1_sc_tmp;
            H1_sc_tmp = [];
                
            H1adj_sc( 1:size_H1_sc_tmp(2), end-size_H1_sc_tmp(1)+1:end ) = H1adj_sc_tmp;
            H1adj_sc_tmp = [];
                
            % interface -> other ribbons
            H1_interface = zeros(size_gfin(1), size_interface );
            H1adj_interface = zeros( size_interface, size_gfin(2) );
                
            H1_interface(end-size_H1_interface_tmp(1)+1:end, 1:size_H1_interface_tmp(2)) = H1_interface_tmp;
            H1_interface_tmp = []; 
                
            H1adj_interface(1:size_H1_interface_tmp(2), end-size_H1_interface_tmp(1)+1:end ) = H1adj_interface_tmp;
            H1adj_interface_tmp = [];                 
            
                
            gfininv = [ gfininv, -H1_interface, zeros(size_gfin(1),size_gfin_tmp(2)); ...
                  -H1adj_interface, WS_ribbon.E*eye(size(H0_interface))-H0_interface, -H1_sc; ...
                zeros(size_gfin_tmp(1),size_gfin(2)), -H1adj_sc, gfininv_tmp]; 
                             

            [coordinates_tmp, coordinates_iterface] = ribbon_tmp.getCoordinates();
            fnames = fieldnames(coordinates);
            for idx = 1:length( fnames )
                fname = fnames{idx};
                if strcmp( fname, 'a' )
                    continue
                else
                    coordinates.(fname) = [coordinates.(fname); coordinates_iterface{1}.(fname); coordinates_tmp.(fname)];
                end
            end
            
                
        end 
        
              
           
        if decimate
            size_gfininv = size(gfininv,1);
            M = Surface_sc.Read('M');
            kulso_szabfokok = [1:M, size_gfininv-M+1:size_gfininv];
            [gfininv, coordinates] = ribbon.DecimationFunction( kulso_szabfokok, gfininv, 'coordinates', coordinates);
        end
            
        
    end
    
              
    
    
    if ~onlygfininverz
        if isnan(rcond(gfininv) ) || abs( rcond(gfininv) ) < 1e-15
            gfin = CommonFunctionsHandle.inv_SVD( gfininv );
        else
            gfin = inv(gfininv);
        end       
        gfininv = [];
    else
        gfin = [];
    end       
        
end



% setEnergy
function setEnergy( Energy )
	ribbon.setEnergy( Energy )
	for idx = 1:length(WS.Ribbons)
		WS.Ribbons{idx}.setEnergy(Energy);
	end
end	



% createInterfaceRegions
function createInterfaceRegions()

    Idx = length(WS.Ribbons) + 1;
    WS.Surface_interface = cell(Idx,1);
    
    % crating the interface region between the leads and the scattring region
    WS.Surface_interface{1} = WS_ribbon.Surface_interface{1};
    WS.Surface_interface{Idx} = WS_ribbon.Surface_interface{2};
    
    % creating the interface regions between the combined ribbons
    for idx = 1:length(WS.Ribbons)
        WS_ribbon_tmp = WS.Ribbons{idx}.GetWorkspace();
        if isempty( WS.Surface_interface{idx+1} )
        	WS.Surface_interface{idx+1} = WS_ribbon_tmp.Surface_tmp.CreateClone();
        end
        WS.Ribbons{idx}.setInterfaceRegions( {WS.Surface_interface{idx}, WS.Surface_interface{idx+1}} ); 
    end
    
    WS_ribbon_tmp = WS.Ribbons{1}.GetWorkspace();
    WS_ribbon.Surface_tmp = WS_ribbon_tmp.Surface_tmp;
    ribbon.DecimateInterface(1);
    WS_ribbon_tmp = WS.Ribbons{end}.GetWorkspace();
    WS_ribbon.Surface_tmp = WS_ribbon_tmp.Surface_tmp;
    ribbon.DecimateInterface(2);
    
    % Decimating the interface regions
    for idx = 2:length(WS.Ribbons)
        WS.Ribbons{idx}.DecimateInterface(1)
    end  
   
        
end



% checking the width of the ribbon
function ret = checkwidth( ribbon_tmp )
    
    % checking if the width of the ribbons is equal
    ws_ribbon_tmp = ribbon_tmp.GetWorkspace();
    if isempty(WS_ribbon.width)
        WS_ribbon.width = ws_ribbon_tmp.width;
        ret = 0;
    elseif WS_ribbon.width ~= ws_ribbon_tmp.width
        ret = -1;
    else
        ret = 0;
    end
        
        
end
   

% Input parsing
    function InputParsing()
        
        p = inputParser;
        p.addParamValue('E', [], @isscalar); %The energy to be used in the ribbons
        p.addParamValue('EF', 0, @isscalar); %The Fermi energy in the leads.
        p.addParamValue('silent', 0);
		p.addParamValue('Ribbons', []);
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_zigzag_leads.xml'], @ischar);
        
        p.parse(varargin{:});

        E            = p.Results.E;
        EF           = p.Results.EF;
        WS.silent    = p.Results.silent; 
		Ribbons   = p.Results.Ribbons;
        filenameIn    = p.Results.filenameIn;
        filenameOut   = p.Results.filenameOut;


        if ~isempty(Ribbons)
             for idx = 1:length(Ribbons)
                 ribbon_tmp = Ribbons{idx};
                 ws_ribbon_tmp = ribbon_tmp.GetWorkspace();
                 
                 if isempty( ribbon )
                     ribbon = Ribbon('E', E, 'EF', EF, 'width', ws_ribbon_tmp.width, 'silent', WS.silent, 'filenameIn', filenameIn, 'filenameOut', filenameOut );
                     WS_ribbon = ribbon.GetWorkspace();
                 else
                     % checking if the width of the ribbons is equal
                     if checkwidth( ribbon_tmp ) == -1
                         err = MException('CombineRibbons:InputParsing', 'widths of the ribbons are not identical.');
                         save('Error_CombineRibbons_InputParsing.mat');
                         throw(err);
                     end
                 end
                 
                 
                 addRibbon( ribbon_tmp );
            end
        else
            err = MException('CombineRibbons:InputParsing', 'WS.Ribbons must contain at least one Ribbon interface.');
            save('Error_CombineRibbons_InputParsing.mat');
            throw(err);            
        end
        
        

    

    end
end
