%%    Favago Transport Programming Interface - Ribbon
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for calculations on a ribbon of finite size.
%%
function handles = Ribbon( varargin )

WS = ws_Ribbon;

Opt = [];
param = [];
InputParsing()


G = []; % Greens function of the scattering region
Ginv = []; % inverse of G

CommonFunctionsHandle = CommonFunctions();


display('Creating ribbon object')


createShape();
setParameters();
parseXML();
setFermiEnergy();
CreateHandles();
% create Hamiltonians and coordinates
CreateRibbon('justHamiltonians', true);


handles.GetWorkspace                = @GetWorkspace;
handles.ShiftLeads                  = @ShiftLeads;
handles.Transport                   = @Transport;
handles.CreateRibbon                = @CreateRibbon;
handles.CreateScatter               = @CreateScatter;
handles.setEnergy                   = @setEnergy;
handles.CalcFiniteGreensFunction    = @CalcFiniteGreensFunction;
handles.GetFiniteGreensFunction     = @GetFiniteGreensFunction;
handles.CustomDysonFunc             = @CustomDysonFunc;
handles.setHandlesForMagneticField  = @setHandlesForMagneticField;
handles.AddPNPotential              = @AddPNPotential;
handles.createPNpotential           = @createPNpotential;
handles.DecimationFunction          = @DecimationFunction;
handles.setInterfaceRegions         = @setInterfaceRegions;
handles.CreateInterface             = @CreateInterface;
handles.DecimateInterface           = @DecimateInterface;
handles.ShiftCoordinates            = @ShiftCoordinates;
handles.getCoordinates              = @getCoordinates;
handles.CreateClone                 = @CreateClone;

%% Transport
% Calculate transport --- use for development pupose only in "contact/scattering center/contact" arrangement.
    function [Conductivity,aspect_ratio,Conductance,ny,DeltaC] = Transport(Energy, varargin)
        setEnergy( Energy )
        
        p = inputParser;
        p.addParamValue('constant_channels', false);
        p.addParamValue('ContactPotInterface', [])
        p.parse(varargin{:});
        constant_channels     = p.Results.constant_channels;
        ContactPotInterface   = p.Results.ContactPotInterface;
        
        CalcFiniteGreensFunction('gauge_trans', true, 'ContactPotInterface', ContactPotInterface);        
        
        Dysonfunc = @()CustomDysonFunc( 'constant_channels', constant_channels);
        try
            WS.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        catch errCause
            err = MException('Ribbon:Transport', 'Error occured in calculating the Greens function');
            err = addCause(err, errCause);
            save('Error_Ribbon_Transport.mat');
            Conductance = NaN;
            Conductivity = NaN;
            aspect_ratio = NaN;
            ny = NaN(2,1);
            DeltaC = NaN;
            return
        end
        [S,ny] = WS.FL_handles.SmatrixCalc();
        
        norma = norm(S*S'-eye(sum(ny)));
            
        if norma >= 1e-3
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if ny(1) ~= ny(2)
           disp( ['openchannels do not match: ',num2str(ny(1)), ' ', num2str(ny(2))] )
        end
        
        if strcmp(param.scatter.End_Type, 'A')
            aspect_ratio = (WS.width*3/2)/((WS.height+3)*sqrt(3));
        else
            aspect_ratio = ((WS.width-0.5)*sqrt(3))/((WS.height+3)*3);
        end
        
        conductance = WS.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(2,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        Conductivity = C/aspect_ratio;
        Conductance = C;
        
        disp( ['aspect ratio = ', num2str(aspect_ratio), ' conductance = ', num2str(C), ' open_channels= ', num2str(ny(1)), ' conductivity = ', num2str(Conductivity)])        
        %2/pi
        
    end


%% getCoordinates
% Gets the coordinates of the surface points of the finite ribbon.
function [coordinates, coordinates_interface] = getCoordinates()
    try
        coordinates = WS.Surface_tmp.Read('coordinates');
        fnames = fieldnames(coordinates);
        for idx = 1:length( fnames )
            fname = fnames{idx};
            if strcmp( fname, 'a' )
                continue
            elseif strcmp( fname, 'x' )
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname) + (WS.height-1)*coordinates.a(1)];
            elseif strcmp( fname, 'y' )
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname) + (WS.height-1)*coordinates.a(2)];
            else
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname)];
            end
        end
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the ribbon from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end
    
    
    try
        if isempty( WS.Surface_interface )
            coordinates_interface = [];
            return
        end
        
        coordinates_interface = cell( length(WS.Surface_interface), 1);
        for idx = 1:length( WS.Surface_interface )
            coordinates_interface{idx} = WS.Surface_interface{idx}.Read('coordinates');
        end
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the interface region from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end    
        
end

%% ShiftCoordinates
%   Shifts the coordinates of the sites in the ribbon by an integer multiple of the lattice vector. The coordinates of the Leads are automatically adjusted later
    function ShiftCoordinates( shift )  
        WS.Surface_tmp.ShiftCoordinates( shift );
        if ~isempty( WS.Surface_interface )
            for idx = 1:length(WS.Surface_interface)
                WS.Surface_interface{idx}.ShiftCoordinates( shift );
            end
        end
        
        WS.shift = WS.shift + shift;
    end

%% CreateScatter
% Creates createH object for the the scattering region.
    function CreateScatter( )       
        Surface_tmp = WS.Surface_tmp.CreateClone();
        Surface_tmp.Reset();
        
        Surface_tmp.CreateHamiltonians( 'toSave', 0);
        Surface_tmp.ShiftCoordinates( WS.shift ); 
            
        if ~isempty(WS.potential)
           potential2apply = WS.potential( WS.Surface_tmp.Read('coordinates'));
           WS.Surface_tmp.AddPotential( potential2apply );
        end
       
        
       	% apply magnetic field in scatter
        if ~isempty( WS.PeierlsTransform_Scatter )
           	display('Applying magnetic field in ribbon Hamiltonians')
           	WS.PeierlsTransform_Scatter.PeierlsTransformLeads( Surface_tmp );            
        end
        createH = CreateHamiltonians(Opt, param);
        createH.CreateScatterH( 'Surface_tmp', Surface_tmp );        
        WS.CreateH = createH;    
        
    end

%% setEnergy
% Sets the energy for the calculations
    function setEnergy( Energy )
        WS.E = Energy;
        WS.FL_handles.setEnergy( WS.E ); 
         
        WS.Surface_tmp.Reset();
        for idx = 1:length(WS.Surface_interface)
        	WS.Surface_interface{idx}.Reset(); 
        end  
        
        CreateRibbon('justHamiltonians', true);
        
    end

%% ShiftLeads
% Shifts the insite energies in the leads by the given energy E
    function ShiftLeads( Energy )
        Surface = WS.FL_handles.Read( 'Surface' );
        
        for idx = 1:length(Surface)
            Surface{idx}.ShiftLead(Energy );
        end
        
    end




%% CustomDysonFunc
% Custom Dyson function for the "contact/scattering center/contact" arrangement.
    function Gret = CustomDysonFunc( varargin )
        
    p = inputParser;
    p.addParamValue('gfininv', []);
    p.addParamValue('Surface_sc', cell(2,1));
    p.addParamValue('constant_channels', true);
    p.addParamValue('onlyGinverz', false );
    p.addParamValue('recalculateSurface', true );
    p.addParamValue('decimate', true );
    p.addParamValue('kulso_szabfokok', []); %The list of sites to be left after the decimation procedure
    p.addParamValue('SelfEnergy', true); %set true to calculate the Dyson equation with the self energy
    p.parse(varargin{:});
    gfininv     = p.Results.gfininv;
    Surface_sc  = p.Results.Surface_sc;
    constant_channels = p.Results.constant_channels;
    onlyGinverz        = p.Results.onlyGinverz;
    recalculateSurface = p.Results.recalculateSurface;  
    decimate           = p.Results.decimate; 
    kulso_szabfokok = p.Results.kulso_szabfokok;
    SelfEnergy         = p.Results.SelfEnergy;
    
    if isempty(gfininv)
        rcond_G = rcond(G);
        if isnan(rcond_G) || abs( rcond_G ) < 1e-15
            gfininv = CommonFunctionsHandle.inv_SVD( G );
        else
            gfininv = inv(G);
        end       
    end
    
    if recalculateSurface
    
        if abs(WS.PNpotential) >= 1e-6
            pot = ribbon.createPNpotential();
        else
            pot = [];
        end
    
        % creating interfaces for the Leads
        if constant_channels
            shiftLeads = ones(length(param.Leads),1)*WS.E;
        else
            shiftLeads = ones(length(param.Leads),1)*0;
        end
    
        % creating objects for Leads
        if isempty( WS.Surface_interface )
            coordinates_shift = [-1, WS.height] + WS.shift;
        else
            coordinates_shift = [-2, WS.height+1] + WS.shift;
        end                    
        Surface = WS.FL_handles.LeadCalc('coordinates_shift', coordinates_shift, 'shiftLeads', shiftLeads, 'potential', pot, ...
            'SelfEnergy', SelfEnergy, 'SurfaceGreensFunction', ~SelfEnergy, 'gauge_field', WS.gauge_field);
    else
        Surface = WS.FL_handles.Read( 'Surface' );
    end
    
    %creating matrices for the Dyson Eq
    Ms = WS.FL_handles.Read('M');
    
    
    if SelfEnergy
        Ginverz = CalcGinverzwithSelfEnergy();
    else
        Ginverz = CalcGinverz();
    end
    
    if decimate
        if isempty(kulso_szabfokok)
            kulso_szabfokok = 1:sum(Ms);
        end
        Ginverz = DecimationFunction( kulso_szabfokok, Ginverz);
    end
    
    if onlyGinverz
        Gret = [];
        return
    end
 	
    rcond_Ginverz = rcond(Ginverz);
    if isnan( rcond_Ginverz )
 		err = MException('Ribbon:CustomDysonFunc', 'NaN is the condition number for Ginverz');
        save('Error_Ribbon_CustomDysonFunc.mat');
        throw(err);
    end
    
    if abs( rcond_Ginverz ) < 1e-15
        display( 'Regularizing Ginverz by SVD',1);
        Gret = CommonFunctionsHandle.inv_SVD( Ginverz );
    else
        Gret = inv( Ginverz );
    end  
    
    
        %-------------------------------
        % calculate the inverse Greens function
        function Ginverz = CalcGinverz()
            
            Gsurfinverz = cell(length(Ms),1);
            H1  = cell(length(Ms),1);
            H1adj = cell(length(Ms),1);
            for idx = 1:length(Ms)
                Gsurfinverz{idx} = Surface{idx}.Read('gsurfinv');%inv(gsurftemp);
                % H1_interface = H1_lead
                H1{idx}  = Surface{idx}.Read('H1'); %WS.Surface_interface{idx}.Read('H1');
                H1adj{idx}  = Surface{idx}.Read('H1adj'); %WS.Surface_interface{idx}.Read('H1adj'); 
            end
    
    
            H_interface = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            if ~isempty( WS.Surface_interface )        
                for idx = 1:length( WS.Surface_interface )
                    if isempty(Surface_sc{idx})
                        Surface_sc{idx} = WS.Surface_tmp.CreateClone();
                        if idx == 1
                            coordinates_shift = -1;
                        elseif idx == 2
                            coordinates_shift = WS.height;
                        end
                        Surface_sc{idx}.ShiftCoordinates( coordinates_shift )
                    end
                    H_interface{idx} = WS.Surface_interface{idx}.Read('H0');
                    if ~isempty( WS.PeierlsTransform_Scatter )                 
                        WS.PeierlsTransform_Scatter.gaugeTransformationOnLead( Surface_sc{idx}, WS.gauge_field );
                    end
                    H1_sc{idx} = Surface_sc{idx}.Read('H1');
                    H1adj_sc{idx} = Surface_sc{idx}.Read('H1adj');
                end        
            end

    
    
            % The Dyson Eq
            GsurfinverzAll = zeros(sum(Ms), sum(Ms));
            H1_All = zeros(sum(Ms), sum(Ms));
            H1adj_All = zeros(sum(Ms), sum(Ms));
            if ~isempty( WS.Surface_interface )
                H_interface_All = zeros(sum(Ms), sum(Ms));
                H1_sc_All = zeros(sum(Ms), sum(Ms));
                H1adj_sc_All = zeros(sum(Ms), sum(Ms));
            end
    
            for idx = 1:length(Ms)
                    GsurfinverzAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = Gsurfinverz{idx};
                    Gsurfinverz{idx} = [];

                    if ~isempty( WS.Surface_interface )
                        H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
                        H1{idx} = [];
                        H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
                        H1adj{idx} = [];
                        H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                        H_interface{idx} = [];
                        H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1_sc{idx};%WS.Surface_interface{idx}.Read('H1');
                        H1adj_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};%WS.Surface_interface{idx}.Read('H1adj');
                    else
                        H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
                        H1{idx} = [];
                        H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
                        H1adj{idx} = [];                
                    end
            end
    
            if ~isempty( WS.Surface_interface )
                Ginv_interface = WS.E*eye(size(H_interface_All)) - H_interface_All;
            end
      
    
            if ~isempty(WS.Surface_interface)
                Ginverz = zeros(3*sum(Ms));
                Ginverz(1:sum(Ms),1:sum(Ms)) = GsurfinverzAll;
                Ginverz(1:sum(Ms),sum(Ms)+1:2*sum(Ms)) = -H1_All;
                Ginverz(sum(Ms)+1:2*sum(Ms), 1:sum(Ms)) = -H1_All';
                Ginverz(sum(Ms)+1:2*sum(Ms), sum(Ms)+1:2*sum(Ms)) = Ginv_interface;
                Ginverz(sum(Ms)+1:2*sum(Ms),2*sum(Ms)+1:3*sum(Ms)) = -H1_sc_All;
                Ginverz(2*sum(Ms)+1:3*sum(Ms), sum(Ms)+1:2*sum(Ms)) = -H1adj_sc_All;    
                Ginverz(2*sum(Ms)+1:3*sum(Ms), 2*sum(Ms)+1:3*sum(Ms)) = gfininv;
                %H1_All
                %Ginv_interface
            else
                Ginverz = [ GsurfinverzAll, -H1_All; -H1_All', gfininv ]; 
            end
            
        end
        
        
        %-------------------------------
        % calculate the inverse Greens function with the self energy
        function Ginverz = CalcGinverzwithSelfEnergy()
            SelfEnergy = cell(length(Ms),1);
            for idx = 1:length(Ms)
                SelfEnergy{idx} = Surface{idx}.Read('Sigma');%inv(gsurftemp);
            end
    
    
            H_interface = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            for idx = 1:length( Ms )
                if isempty(Surface_sc{idx})
                    Surface_sc{idx} = WS.Surface_tmp.CreateClone();
                    if idx == 1
                        coordinates_shift = -1;
                    elseif idx == 2
                        coordinates_shift = WS.height;
                    end
                    Surface_sc{idx}.ShiftCoordinates( coordinates_shift )
                end
                if ~isempty( WS.PeierlsTransform_Scatter )                 
                    WS.PeierlsTransform_Scatter.gaugeTransformationOnLead( Surface_sc{idx}, WS.gauge_field );
                end
                if ~isempty( WS.Surface_interface ) 
                    H_interface{idx} = WS.Surface_interface{idx}.Read('H0');
                    H1_sc{idx} = Surface_sc{idx}.Read('H1');
                    H1adj_sc{idx} = Surface_sc{idx}.Read('H1adj');
                else
                    H_interface{idx} = Surface{idx}.Read('H0');
                    H1_sc{idx} = Surface{idx}.Read('H1');
                    H1adj_sc{idx} = Surface{idx}.Read('H1adj');
                end
                
        
            end  

            % The Dyson Eq
            SelfEnergyAll = zeros(sum(Ms), sum(Ms));
            H_interface_All = zeros(sum(Ms), sum(Ms));
            H1_sc_All = zeros(sum(Ms), sum(Ms));
            H1adj_sc_All = zeros(sum(Ms), sum(Ms));
    
            for idx = 1:length(Ms)
                    SelfEnergyAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = SelfEnergy{idx};
                    SelfEnergy{idx} = [];

                    H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                    H_interface{idx} = [];
                    H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1_sc{idx};%WS.Surface_interface{idx}.Read('H1');
                    H1_sc{idx} = {};
                    H1adj_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};%WS.Surface_interface{idx}.Read('H1adj');
                    H1adj_sc{idx} = [];
            end
    
            Ginv_interface = WS.E*eye(size(H_interface_All)) - H_interface_All;
            H_interface_All = [];

            Ginverz = zeros(2*sum(Ms));
            Ginverz(1:sum(Ms),1:sum(Ms)) = Ginv_interface - SelfEnergyAll;
            Ginverz(1:sum(Ms),sum(Ms)+1:2*sum(Ms)) = -H1_sc_All;
            Ginverz(sum(Ms)+1:2*sum(Ms), 1:sum(Ms)) = -H1adj_sc_All;    
            Ginverz(sum(Ms)+1:2*sum(Ms), sum(Ms)+1:2*sum(Ms)) = gfininv;    
            
            
            
        end        
        
    end



%% DecimationFunction
% Performs the decimation procedure on the inverse Green's Function
    function [ret, coordinates_ret] = DecimationFunction( kulso_szabfokok, ginv, varargin)
        p = inputParser;
        p.addParamValue('coordinates', [] );        
        p.parse(varargin{:});       
        coordinates    = p.Results.coordinates;
        
        CreateH = CreateHamiltonians(Opt, param);
        
        Hamiltoni = eye(size(ginv))*WS.E - ginv;
        ginv = [];
    
        CreateH.Write('Hscatter', Hamiltoni);
        clear Hamiltoni;
    
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', coordinates);
        CreateH.Write('HamiltoniansCreated', 1);
        CreateH.Write('HamiltoniansDecimated', 0);
        
        Decimation_Procedure = Decimation(Opt); 
        Decimation_Procedure(WS.E, CreateH, 'Hscatter', 'kulso_szabfokok');
    
        Hamiltoni = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');
    
        ret = eye(size(Hamiltoni))*WS.E - Hamiltoni;
        Hamiltoni = [];
        
        
        coordinates_ret = CreateH.Read('coordinates');
        
        
    end


%% GetFiniteGreensFunction
% Read out the calculated surface Greens fuction of a finite ribbon.
    function [Gret, Ginvret] = GetFiniteGreensFunction()
        if isempty(G) && isempty(Ginv)
            CalcFiniteGreensFunction()
        end
        
        Gret = G;
        Ginvret = Ginv;
    end

%% CalcFiniteGreensFunction
% Calculates the surface Greens fuction for a finite ribbon.
    function CalcFiniteGreensFunction( varargin )
    
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians     
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('onlyGinv', false);
        p.parse(varargin{:});
        gauge_trans     = p.Results.gauge_trans;
        ContactPotInterface  = p.Results.ContactPotInterface;
        onlyGinv        = p.Results.onlyGinv;
        
      	CreateRibbon( 'ContactPotInterface', ContactPotInterface);        

        z1 = 0;
        z2 = WS.height;   
        
        G = [];
        Ginv = [];
        
        WS.Surface_tmp.FiniteGreenFunction(z1,z2, 'onlygfininv', true);        
        Ginv = WS.Surface_tmp.Read( 'gfininv' ); 
        
        % coordinates
        coordinates_scatter = getCoordinates();
           
           
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans 
            try
            	% gauge transformation on Green's function
                if ~isempty(Ginv) && ~isempty(WS.PeierlsTransform_Scatter)
                    % gauge transformation on the inverse Green's function
                    Ginv = WS.PeierlsTransform_Scatter.gaugeTransformation( Ginv, coordinates_scatter, WS.gauge_field );
                end
            catch  errCause  
                err = MException('Ribbon:CalcFiniteGreensFunction', 'Unable to perform gauge transformation');
                err = addCause(err, errCause);
                save('Error_Ribbon_CalcFiniteGreensFunction.mat')
                throw(err);
            end
        end
        
        if ~onlyGinv
            rcond_Ginv = rcond(Ginv);
            if isnan(rcond_Ginv ) || abs( rcond_Ginv ) < 1e-15
                display( 'Regularizing Ginv by SVD',1);
                G = CommonFunctionsHandle.inv_SVD( Ginv );
            else
                G = inv(Ginv);
            end       
            Ginv = [];
        else
            G = [];
        end
        
    end


%% CreateRibbon
% Creates the Hamiltonians for the ribbon shaped scattering region.
    function CreateRibbon( varargin )
    
        p = inputParser;
        p.addParamValue('justHamiltonians', false); 
        p.addParamValue('ContactPotInterface', []); 
        p.addParamValue('forcedInterface', false); 
        p.parse(varargin{:});
        justHamiltonians     = p.Results.justHamiltonians;
        ContactPotInterface  = p.Results.ContactPotInterface;
        forcedInterface      = p.Results.forcedInterface;
        
        
                
        
        if ~WS.Surface_tmp.Read( 'HamiltoniansCreated' )
        	display( 'Creating ribbon Hamiltonian.' )
        	WS.Surface_tmp.CreateHamiltonians( 'toSave', 0);
        	WS.Surface_tmp.ShiftCoordinates( WS.shift ); 
            
            if ~isempty(WS.potential)
               potential2apply = WS.potential( WS.Surface_tmp.Read('coordinates'));
               WS.Surface_tmp.AddPotential( potential2apply );
            end
        end
       
        
       	% apply magnetic field in scatter
        if ~isempty( WS.PeierlsTransform_Scatter )
           	display('Applying magnetic field in ribbon Hamiltonians')
           	WS.PeierlsTransform_Scatter.PeierlsTransformLeads( WS.Surface_tmp );            
       	end
                
        % interface region is needed only when decimation is necessary in the ribbon hamiltonians to get invertable H1
        M = WS.Surface_tmp.Read('M');
        H0_size = size( WS.Surface_tmp.Read('H0'), 1);
        if M == H0_size && (~forcedInterface)
            WS.Surface_interface = [];
        end       
        
        
        for idx = 1:length(WS.Surface_interface)
			CreateInterface( idx, 'ContactPotInterface', ContactPotInterface );
        end
        
         
        
        %WS.Surface_tmp.CalcSpektrum('ka_min', -pi, 'ka_max', pi);
        if justHamiltonians
            return
        end

        for idx = 1:length(WS.Surface_interface)
            DecimateInterface( idx );
        end
        
        display( 'Solving Hamiltonians in the scattering region' )        
        Decimation_Procedure_Lead = Decimation(Opt, 'ForcedDecimation', Opt.Decimation);
        Decimation_Procedure_Lead(WS.E, WS.Surface_tmp, 'Hamiltonian2Dec', 'kulso_szabfokok');  
        
        % Trukkos sajatertek
        display(['Eigenvalues for the scattering region'])
        WS.Surface_tmp.TrukkosSajatertekek(WS.E);
        % group velocity
        display(['Group velocity for the scattering region'])
        WS.Surface_tmp.Group_Velocity();
        
	
    end



%% setInterfaceRegions
% Replace the attribute Surface_interface of class handle WS_Ribbon to the custom value
    function setInterfaceRegions( Surface_interface )
       if length( Surface_interface ) ~= 2
           err = MException('Ribbon:setInterfaceRegions', 'The length of Surface_interface must be 2.');
           save('Error_Ribbon_setInterfaceRegions.mat');
           throw(err);          
       end
       
       if ~iscell( Surface_interface )
           err = MException('Ribbon:setInterfaceRegions', 'The Surface_interface must be a cell containing interface Surface_Green_function.');
           save('Error_Ribbon_setInterfaceRegions.mat');
           throw(err);          
       end 
       
       WS.Surface_interface = Surface_interface;
        
    end

%% CreateInterface
% Creates the Hamiltonians for the interface regions
	function CreateInterface( idx, varargin )

        p = inputParser;
        p.addParamValue('ContactPotInterface', []);
        p.parse(varargin{:});
        ContactPotInterface  = p.Results.ContactPotInterface;

		coordinates_shift = [-1, WS.height ] + WS.shift;
        if ~WS.Surface_interface{idx}.Read( 'HamiltoniansCreated' )        		
        	display('Creating interface Hamiltonians')
            WS.Surface_interface{idx}.CreateHamiltonians( 'toSave', 0);
            WS.Surface_interface{idx}.ShiftCoordinates( coordinates_shift(idx) );
        end
        if ~isempty(ContactPotInterface) && ~WS.Surface_interface{idx}.Read( 'HamiltoniansDecimated' )
            H0 = WS.Surface_interface{idx}.Read('H0');
            coordinates = WS.Surface_interface{idx}.Read('coordinates');
            H0 = H0 + sparse(1:size(H0,1), 1:size(H0,1), ContactPotInterface( coordinates.x, coordinates.y ), size(H0,1), size(H0,1));
            WS.Surface_interface{idx}.Write('H0', H0);
        end
        % apply magnetic field in the interface region
        if ~isempty( WS.PeierlsTransform_Leads )            
          	display(['Applying magnetic field in the Hamiltonians of interface region ', num2str(idx)])
            if ~WS.Surface_interface{idx}.isSuperconducting()
               	WS.PeierlsTransform_Leads.PeierlsTransformLeads( WS.Surface_interface{idx} );  
            else
               	WS.PeierlsTransform_Leads.gaugeTransformationOnLead( WS.Surface_interface{idx}, WS.gauge_field );
            end
        end  
        
	end


%% DecimateInterface
% Decimates the interface region (between the leads and the scattering region) for connecting the leads to scattering center (needed in hexagonal lattice).
    function DecimateInterface( idx, varargin )
        
        p = inputParser;
        p.addParamValue('E', WS.E);
        p.parse(varargin{:});
        E  = p.Results.E;
    
    	if WS.Surface_interface{idx}.Read( 'HamiltoniansDecimated' )
    		display('Hamiltonians already decimated');
    		return
   		end
    
        if ~WS.Surface_interface{idx}.Read( 'HamiltoniansCreated' )
            err = MException('Ribbon:DecimateInterface', 'Hamiltonians in Surface_tmp are not created. Thus, unable to perform the decimation.');
            save('Error_Ribbon_DecimateInterface.mat');
            throw(err); 
        end    
    
        H = WS.Surface_interface{idx}.Read('Hamiltonian2Dec');          
        H0 = WS.Surface_tmp.Read('H0');


               
        % tranform the gauge in H0 if magnetic field is used
        if ~isempty( WS.PeierlsTransform_Scatter )
            %coordinates_tmp = WS.Surface_interface{idx}.Read('coordinates');%WS.Surface_tmp.Read( 'coordinates' );
            coordinates_tmp = WS.Surface_tmp.Read( 'coordinates' );
            if idx == 2
                coordinates_tmp.y = coordinates_tmp.y + (WS.height)*coordinates_tmp.a(2);
                coordinates_tmp.x = coordinates_tmp.x + (WS.height)*coordinates_tmp.a(1);
            end
                
            H0 = WS.PeierlsTransform_Scatter.gaugeTransformation( H0, coordinates_tmp, WS.gauge_field );
        end
        
        % Hscattering -> H1 -> Hinterface     
        Hsize = size(H,1);
        H( Hsize/2+1:Hsize, Hsize/2+1:Hsize ) = H0;
        %H( 1:Hsize/2, 1:Hsize/2 ) = H0;
        
        kulso_szabfokok = WS.Surface_interface{idx}.Read('kulso_szabfokok');
        coordinates = WS.Surface_interface{idx}.Read('coordinates');
        
        if length(kulso_szabfokok) == size(H,1)
            %nothing to decimate
            return
        end
        
        
        CreateH = CreateHamiltonians(Opt, param);
        %CreateH = WS.CreateH;        
        CreateH.Write('Hscatter', H);
        H = [];
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', coordinates);
        CreateH.Write('HamiltoniansCreated', 1);
        CreateH.Write('HamiltoniansDecimated', 0);

        Decimation_Procedure = Decimation(Opt); 
        Decimation_Procedure(E, CreateH, 'Hscatter', 'kulso_szabfokok');
        
        H = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');    
        
        
        coordinates = CreateH.Read('coordinates');
        WS.Surface_interface{idx}.Write('coordinates', coordinates);
        WS.Surface_interface{idx}.Write('Hamiltonian2Dec', H); 
        WS.Surface_interface{idx}.Write('HamiltoniansDecimated', CreateH.Read('HamiltoniansDecimated')); 
    end




%% setHandlesForMagneticField
% Sets the function handles of the vector potential and gauge transformation.
    function setHandlesForMagneticField( varargin )     
    
        p = inputParser;
        p.addParamValue('scatter', [] );
        p.addParamValue('lead', [] );
        p.addParamValue('gauge_field', [] ); 
        
        p.parse(varargin{:});

       
        hscatter    = p.Results.scatter;
        hlead       = p.Results.lead;
        WS.gauge_field = p.Results.gauge_field;
        
        
        display('Adding handles of the magnetic field to the scattering region') 
        if Opt.magnetic_field && isempty( WS.PeierlsTransform_Scatter )
        	WS.PeierlsTransform_Scatter = Peierls(Opt, 'Vectorpotential', hscatter);
        elseif ~isempty( WS.PeierlsTransform_Scatter ) && ~isempty(hscatter)
        	WS.PeierlsTransform_Scatter.setVectorPotential( hscatter );
        end
        
        if Opt.magnetic_field && isempty( WS.PeierlsTransform_Leads )
        	WS.PeierlsTransform_Leads   = Peierls(Opt, 'Vectorpotential', hlead);
        elseif ~isempty( WS.PeierlsTransform_Leads ) && ~isempty(hlead)
        	WS.PeierlsTransform_Leads.setVectorPotential( hlead );
       	end
       	
    end


%% AddPNPotential
% Adds the p-n potential to the ribbon Hamiltonian
    function AddPNPotential( Surface_temp )
        if abs( WS.PNpotential ) <= 1e-6
            return
        end
        
        disp( 'Adding pn potential to the ribbon Hamiltonian' )
        H0 = Surface_temp.Read('H0');
        
        if 2*WS.width ~= size(H0,1)
            disp('WS.width ans size(H0) are in mismatch')
            return
        end
        pot = createPNpotential();
        Surface_temp.AddPotential( pot );
        
    end
        
        
        
%% createPNpotential
% Creates the p-n potential to be applied in the Hamiltonians
    function pot = createPNpotential()
        deltax_ps = 2;
        deltax_pt = 1; 
        deltax    = 3;
    
        pot = zeros(size(2*WS.width,1));
            
        xcoords1 = 1:WS.width;
        indexekpt = logical( mod(xcoords1,2)==1 );
        indexekps = ~indexekpt;            
        xcoords1( indexekpt ) = ( xcoords1( indexekpt )-1 )*deltax/2;
        xcoords1( indexekps ) = ( xcoords1( indexekps )-2 )*deltax/2 +deltax_pt;
            
        xcoords2 = 1:WS.width;   
        indexekpt = logical( mod(xcoords2,2)==1 );
        indexekps = ~indexekpt;     
        xcoords2( indexekpt ) = ( xcoords2( indexekpt )-1 )*deltax/2 - deltax_pt/2;
        xcoords2( indexekps ) = ( xcoords2( indexekps )-2 )*deltax/2 + deltax_ps - deltax_pt/2;
            
                                                              
        eta = WS.PNeta;    
        a = 2*WS.PNpotential/(max(xcoords1) - min(xcoords1))/eta;
        b = -WS.PNpotential/eta;
            
        pot( 1:WS.width ) = a*xcoords1 + b;
        pot( WS.width+1:2*WS.width ) = a*xcoords2 + b;
        
        indexek = logical(abs(pot) >= WS.PNpotential);
        pot(indexek) = sign(pot(indexek))*abs(WS.PNpotential);
            
            
	
    end


%% CreateClone
%   Creates a clone of the current interface.
    function ret = CreateClone()
        
        ret = Ribbon( 'width', WS.width, ...
            'height', WS.height, ...
            'filenameIn', WS.filenameIn, ...
            'filenameOut', WS.filenameOut, ...
            'E', WS.E, ...
            'EF', WS.EF, ...
            'phi', WS.phi, ...
            'silent', WS.silent, ...
            'potential', WS.potential, ...
            'PNpotential', WS.PNpotential, ...
            'PNeta', WS.PNeta, ...
			'Opt', Opt, ...
			'param', param, ...
			'tree', WS.tree );
        
        WS_ret = ret.GetWorkspace();
        WS_ret.CreateH = WS.CreateH.CreateClone();
        WS_ret.FL_handles = WS.FL_handles.CreateClone();
        WS_ret.Surface_tmp = WS.Surface_tmp.CreateClone();
        WS_ret.Surface_interface = cell(size(WS.Surface_interface));
        for idx = 1:length(WS.Surface_interface)
            WS_ret.Surface_interface{idx} = WS.Surface_interface{idx}.CreateClone();
        end
        if ~isempty( WS.PeierlsTransform_Scatter )
            WS_ret.PeierlsTransform_Scatter = WS.PeierlsTransform_Scatter.CreateClone();
        end
        
        if ~isempty( WS.PeierlsTransform_Leads )
            WS_ret.PeierlsTransform_Leads   = WS.PeierlsTransform_Leads.CreateClone();    
        end
        WS_ret.gauge_field              = WS.gauge_field;  % function handle for the scalar field to transform the vector potential from Landauy to Landaux
        
    end   
      

%% GetWorkspace()
% Return the handle class to the workspace of the given interface. Changes in fields of the workspace are not always safe, since reinitialization of function handles are not performed.
    function ret = GetWorkspace()
        ret = WS;        
    end

% setsFermiEnergy
% Sets the Fermi energy for the calculations on the sites.
    function setFermiEnergy()
        if ~isempty(WS.EF)
            param.scatter.epsilon = param.scatter.epsilon - WS.EF;
            for idx = 1:length(param.Leads)
                param.Leads{idx}.epsilon = param.Leads{idx}.epsilon - WS.EF;
            end
       end    
		WS.EF = [];    
    end

% Creates handles for geometry operations
    function CreateHandles()  
        
        setHandlesForMagneticField();
        
        WS.CreateH = CreateHamiltonians(Opt, param);
        WS.FL_handles = Transport_Interface(WS.E, Opt, param, 'CreateH', WS.CreateH, 'PeierlsTransform', WS.PeierlsTransform_Leads );
        Opt = WS.FL_handles.Read( 'Opt' );
        WS.Surface_tmp = WS.FL_handles.SurfaceGreenFunctionCalculator([], 'createCore', 1);
        WS.Surface_interface = cell(length(param.Leads),1);
        for idx = 1:length(WS.Surface_interface)
            WS.Surface_interface{idx} = WS.FL_handles.SurfaceGreenFunctionCalculator(idx, 'createCore', 1);
        end
        
    end



    % Opens the XML file and fills up the parameter structures
    function parseXML()  
		if isempty( WS.tree )      
	        WS.tree = xmlread(WS.filenameIn); 
            traverseNodes( WS.tree );  
		end
        xmlwrite(WS.filenameOut,WS.tree);
    end


    % Parses over the attributes of the tree node
    function traverseNodes( theNode )
        % Recurse over node children.
        
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        

        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'parameters' )
                traverseNodes( theChild )
            elseif strcmp(char(theChild.getNodeName), 'Scatter_region' )
                setScatterNode( theChild ) 
            elseif strcmp(char(theChild.getNodeName), 'Lead_parameters' )
                setLeadParameters( theChild )
            end
            
        end
    end



% Sets a value for a node in the xml tree under a specified node. If the node does not exit, this function creates it.
    function setNode( anchorNode, nodename, value, valueattribute)
        
        if ~exist('valueattribute', 'var')
            valueattribute = 'value';
        end
        
        Item = anchorNode.getElementsByTagName( nodename ).item(0);    
        
        if isempty( Item )
            Item = WS.tree.createElement( nodename );
            Item.setAttribute(valueattribute, num2str( value ));
            anchorNode.appendChild(Item);
        else
            Item.setAttribute(valueattribute, num2str( value ));
        end        
        
        
    end
        
    % Sets parameters for Leads
    function setLeadParameters( theNode )
        
        setNode( theNode, 'NofLeads', 2);
        
        %return
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'lead' )
                
                attributes = parseAttributes(theChild);
                
                if isfield(attributes, 'num')
                    idx = str2double(attributes.num);
                else
                    idx = idx + 1;
                    pointNode.theChild('num', num2str(idx));
                end
                
                if ~isempty(WS.EF)  
                    setNode( theChild, 'epsilon', param.Leads{idx}.epsilon - WS.EF);   
                end
                
                if isfield( param.Leads{idx}, 'pair_potential' )
                	setNode( theChild, 'pair_potential', angle(param.Leads{idx}.pair_potential), 'phase');  
               	end
                
                
                
                
            end
            
        end
        
    end


    % Fills up data from one lead node
    function setScatterNode( theNode )
           
        if ~isempty(WS.EF)
            setNode( theNode, 'epsilon', -WS.EF);   
        end       
        
        
        ShapeItem=theNode.getElementsByTagName('shape').item(0);  
        
        if isempty( ShapeItem )
            ShapeItem = WS.tree.createElement( 'shape' );
            theNode.appendChild(ShapeItem);
        end
        
        
        % creating shape node
        display('Creating shape node');
        setNode( ShapeItem, 'width', param.scatter.shape.width);
        setNode( ShapeItem, 'height', param.scatter.shape.height);
        
        return
        
 
        
    end


    % Parses the attributes of the XML node
    % ----- Local function PARSEATTRIBUTES -----
    function attributes = parseAttributes(theNode)
    % Create attributes structure.

    attributes = [];
    if theNode.hasAttributes
        theAttributes = theNode.getAttributes;
        numAttributes = theAttributes.getLength;
        attributes = struct();
    else
        return
    end

    for count = 1:numAttributes
        attrib = theAttributes.item(count-1);
        attributes.( char(attrib.getName) ) = char(attrib.getValue);
    end
    
    end

% creates graphene lead attach points
    function calculate_lead_attach_points()
        for idx = 1:length(param.Leads)
            param.Leads{idx}.M = param.scatter.shape.width;
        end
    end


% create the ribbon shape
    function createShape()
        
        if ~isempty( WS.width ) && ~isempty( WS.height )
            param.scatter.shape.width = WS.width;
            param.scatter.shape.height = WS.height;
        end
        
        if ~isempty(param.scatter.shape.width)
            WS.width = param.scatter.shape.width;
        else
            err = MException('Ribbon:createShape', 'Shape is not given correctly, width is missing');
            throw(err)
        end
        
        if ~isempty(param.scatter.shape.height)
            WS.height = param.scatter.shape.height;
        else
            err = MException('Ribbon:createShape', 'Shape is not given correctly, height is missing');
            throw(err)
        end
        
        calculate_lead_attach_points();
       
    end


% Sets computing parameters
    function setParameters()
        
        if isvector(WS.phi) && length(WS.phi) == length(param.Leads)
            for idx = 1:length(param.Leads)
                param.Leads{idx}.pair_potential = abs(param.Leads{idx}.pair_potential)*exp(1i*WS.phi(idx));
            end
        end
    end



% displays output messages on the screen
    function display( message, nosilent )
        if ~WS.silent || exist('nosilent', 'var')
            disp( message )
        end
       
        if Opt.debug
            fid = fopen('debug.txt', 'a');
            fprintf(fid, [message,'\n']);
            fclose(fid);
        end
        
    end

% Input parsing
    function InputParsing()
    
        p = inputParser;
        p.addParamValue('width', WS.width, @isscalar);
        p.addParamValue('height', WS.height, @isscalar);
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_running_parameters.xml'], @ischar);
        p.addParamValue('E', WS.E, @isscalar);
        p.addParamValue('EF', WS.EF);
        p.addParamValue('phi', WS.phi);  
        p.addParamValue('silent', WS.silent);   
        p.addParamValue('potential', WS.potential);
        p.addParamValue('PNpotential', WS.PNpotential, @isscalar);
        p.addParamValue('PNeta', WS.PNeta, @isscalar);     
        p.addParamValue('Opt', []);
        p.addParamValue('param', []); 
        p.addParamValue('tree', []); 
        
        p.parse(varargin{:});

       
        WS.width     = p.Results.width;
        WS.height    = p.Results.height;
        WS.filenameIn   = p.Results.filenameIn;
        WS.filenameOut  = p.Results.filenameOut;
        WS.E            = p.Results.E;
        WS.EF           = p.Results.EF;
        WS.phi          = p.Results.phi;
        WS.silent       = p.Results.silent;
        WS.potential   = p.Results.potential;
        WS.PNpotential  = p.Results.PNpotential;
        WS.PNeta  = p.Results.PNeta; 
		Opt             = p.Results.Opt;
		param           = p.Results.param;       
		WS.tree         = p.Results.tree; 
        
%         if mod(WS.width,2) == 1 && WS.width ~= 1
%             display('correcting width to be even')
%             WS.width = WS.width + 1;
%         end
                
		if isempty(Opt) || isempty(param)   
			[ Opt, param ] = parseInput( WS.filenameIn );
		end

    end



end
