%%    Favago Transport Programming Interface - SNSJosephson
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for the calculations of the DC Josephson effect.
%%
function handles = SNSJosephson( varargin )

WS = ws_SNSJosephson;

k_B = 8.6173324e-5; % Boltzmann constant in eV/K
beta = [];

ribbon             = [];
WS_ribbon          = [];
ribbon_undecimated = [];
BandWidth          = [];
useSelfEnergy      = true;
ContactPotInterface = [];


CommonFunctionsHandle = CommonFunctions();


InputParsing();




handles.ribbon                        = ribbon;
handles.Transmission                  = @Transmission;
handles.CurrentCalc_continuum         = @CurrentCalc_continuum;
handles.CurrentCalc_discrete          = @CurrentCalc_discrete;
handles.CreateHandlesForMagneticField = @CreateHandlesForMagneticField;
handles.getBandWidth                  = @getBandWidth;
handles.createCurrentOperator         = @createCurrentOperator;
handles.create_scatter_GreensFunction = @create_scatter_GreensFunction;
handles.GetWorkspace                  = @GetWorkspace;


%% Transmission
% Calculates the transmission of the BdG quasiparticles in the arrangement "SC contact/scattering center/SC contact".
    function [C,ny,DeltaC] = Transmission( E, B )
        
        if exist('B', 'var')
            WS.B = B;
        else
            WS.B = 1e-10;
        end        
        
        CreateHandlesForMagneticField( )
        
        if exist('E', 'var')
            SetEnergy( E )
        end
        
        
        gfininv = create_scatter_GreensFunction('onlyAtEnds', true, 'gauge_trans', true);
        recalculateSurface = true;
        %Dysonfunc = @()CustomDysonFunc();
        Dysonfunc = @()ribbon.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', false, 'gfininv', gfininv, 'constant_channels', false);
        WS_ribbon.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        
        
        [S,ny] = WS_ribbon.FL_handles.SmatrixCalc();
        norma = norm(S*S'-eye(sum(ny)));
        if norma >= 1e-3
            display( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        ny
        if ny(1) ~= ny(2)
           display( ['openchannels do not match: ',num2str(ny)] )
        end
        
        
        conductance = WS_ribbon.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(1,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        
        display( ['E = ', num2str(E), ' conductance = ', num2str(C)])
        
    end



%% CurrentCalc_discrete
% Calculates the Josephson current of the discrete bound states.
    function currentvec = CurrentCalc_discrete( DeltaPhi_vec, varargin )
        
        p = inputParser;
        p.addParamValue('range', 'ABS' ); % NBS: normal bound states, ABS: andreev bound states, CONT: scattering range, ALL: all contributions at once
		p.addParamValue('Edb', 511 ); % number of energy points on the contour 
        
        p.parse(varargin{:});       
        range   = p.Results.range;
		Edb     = p.Results.Edb;
    
        if abs(WS.T) <= 1e-6
    		% for zero temperature utilize the E -> -E symmetry
    		fact = 2;
    	else
    		fact = 1;
        end
        
        currentvec = zeros(size(DeltaPhi_vec));
        
        if strcmp( range, 'NBS' )
            display('Calculating current for the normal bound states.', 1)
            getBandWidth();
            Emax = -BandWidth.Lead.Emax;
            Emin = -BandWidth.Scatter.Emax;
            phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;
            %fact = -fact; %the contour integral is in the opposite direction as for the ABS
        elseif strcmp( range, 'ABS' )
            display('Calculating current for the Andreev bound states.', 1)
            getBandWidth();
            param = WS_ribbon.FL_handles.Read( 'param' );         
            pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
            Emin = -pair_potential;
            Emax = -0;
            phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;
        elseif strcmp( range, 'CONT' )
            display('Calculating current for the scattering states.', 1)
            getBandWidth();
            param = WS_ribbon.FL_handles.Read( 'param' );         
            pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
            Emin = -BandWidth.Lead.Emax;
            Emax = -pair_potential;
            phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;    
        elseif strcmp( range, 'ALL' )
            display('Calculating current for all the states.', 1)
            getBandWidth();        
            Emin = min([-BandWidth.Lead.Emax,-BandWidth.Scatter.Emax]);
            Emax = -0;
            phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;                      
        else
            err = MException('SNSJosephson:CurrentCalc_discrete', 'Unrecognized energy range');
            save('Error_SNSJosephson_CurrentCalc_discrete.mat');
            throw(err); 
        end
        
        if Emin >= Emax
            return
        end
        
        display(['Calculating the current between Emin = ', num2str(Emin),' and Emax = ',num2str(Emax),],1);
        
        radius = abs( Emax-Emin) /2;            
        b = min(1, 1/radius);
        Evec = radius*(cos(phivec) + b*1i*sin(phivec)) - radius + Emax;
		deltaEvec = [0, diff(Evec)];		
		currentsurf = zeros( length(Evec), length(DeltaPhi_vec) );  

        if exist('parpool' )
            Opt = WS_ribbon.FL_handles.Read('Opt');
            if ~isempty(Opt.workers)
               	poolobj = parpool(Opt.workers);
            else
               	poolobj = parpool();
            end
        else
           	poolobj = [];
        end
        		
        hdisplay = @display;
        hSetEnergy = @SetEnergy;
        hcreateCurrentOperator = @createCurrentOperator;
        hincreasePhaseDifference = @increasePhaseDifference;
        hcomplexCurrent = @complexCurrent;

        parfor iidx = 1:length(Evec)
tic		
            WS_ribbon = ribbon.GetWorkspace();

            feval( hdisplay, ['Currently evaluating ', num2str(iidx), ' from ', num2str(length(Evec))], 1);
            ribbon_loc = ribbon.CreateClone();
            
			recalculateSurface = true;
            feval(hSetEnergy, Evec(iidx), 'ribbon', ribbon_loc );
            
            [current_op, gfininv] = feval(hcreateCurrentOperator, 'ribbon', ribbon_loc);
            if isnan(current_op)
            	currentsurf( iidx, : ) = NaN;
            	continue
            end	 
            	
            currentvec2int = zeros( size( DeltaPhi_vec ) );
            for jjdx = 1:length( DeltaPhi_vec )                
                try
                	feval(hincreasePhaseDifference, DeltaPhi_vec(jjdx)-DeltaPhi_vec(1), ribbon_loc );
                	currentvec2int( jjdx ) = feval(hcomplexCurrent, ribbon_loc, gfininv, current_op, recalculateSurface);
                catch errCause
                	feval(hdisplay, 'Error in SNSJosephson:CurrentCalc_discrete caused by:');
                	feval(hdisplay, errCause.identifier );
					feval(hdisplay, errCause.stack(1) );
                	feval(hdisplay, 'Giving NaN for the calculated current'); 
					throw(errCause)
                end
                recalculateSurface = false;
            end 
            currentsurf( iidx, : ) = currentvec2int;
toc
        end

        if ~isempty(poolobj)
            delete( poolobj )
        end
		
		% calculating the contour integrals
        for jjdx = 1:length( DeltaPhi_vec )
			currentvec2integrate = currentsurf(:,jjdx);
			% omitting NaNs by substituting interpolates
            indexes = isnan(currentvec2integrate); 
			indexesall = 1:length(currentvec2integrate);
			        
            currentvec2integrate(indexes) = interp1(indexesall(~indexes), currentvec2integrate(~indexes), indexesall(indexes), 'spline' );
			current = fact*imag( SimphsonInt( currentvec2integrate.*transpose(deltaEvec) ))/(pi);
			currentvec( jjdx ) = current;	
        end	
        
        display('Process finished.', 1)


		%------------------------------------------------
		% calculates current for complex energy
		function current = complexCurrent( ribbon_loc, gfininv, current_op, recalculateSurface )
            
            WS_ribbon_loc = ribbon_loc.GetWorkspace();          
                        
            % determining the necessary sites
            if isempty( WS_ribbon_loc.Surface_interface ) || useSelfEnergy
            	% in case no interface region is included
                kulso_szabfokok = size(gfininv,1) + (1:size(gfininv,1));
            else
                % with interface region included in the setup
                kulso_szabfokok = 2*size(gfininv,1) + (1:size(gfininv,1));
            end				
            
            G = ribbon_loc.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', true, 'kulso_szabfokok', kulso_szabfokok, 'gfininv', gfininv, 'constant_channels', false, ...
                'SelfEnergy', useSelfEnergy);
%             G = ribbon_loc.CustomDysonFuncSelfEnergy('recalculateSurface', recalculateSurface, ...
%                 'decimate', true, 'kulso_szabfokok', kulso_szabfokok, 'gfininv', gfininv, 'constant_channels', false);
%             
			G_size = size(G);							
			gfininv_size = size(gfininv);				
			if G_size(1) ~= gfininv_size(1)
				warning( 'wrong size of Greens function' )
				current = 0;
				return
			end
			current = trace( current_op*G );
			 
		end
      
    end

%% CurrentCalc_continuum
% Calculates the Josephson current of the continous scattering states.
    function [currentvec, current_surf] = CurrentCalc_continuum( DeltaPhi_vec, varargin  )
        disp('Calculating current in the continuum range from the scattering matrix.')
        p = inputParser;
		p.addParamValue('Edb', 511 ); % number of energy points on the contour 
        p.parse(varargin{:});       
		Edb     = p.Results.Edb;


    	if abs(WS.T) <= 1e-6
    		% for zero temperature utilize the E -> -E symmetry
    		fact = 2;
    	else
    		fact = 1;
    	end
    	
    	getBandWidth()
        Emax = -BandWidth.Lead.Emin;
        Emin = -BandWidth.Lead.Emax;
        Evec = (0.5+0.5*sin((0:Edb-1)/(Edb-1)*pi - pi/2))*(Emax-Emin) + Emin;
        deltaEvec = [0, diff(Evec) ];
        
        current_surf = zeros(length(DeltaPhi_vec), length(Evec));
        
        hCurrentCalc_continuum_forGivenEnergy = @CurrentCalc_continuum_forGivenEnergy;
        parfor kdx = 1:length( Evec )
            ribbon_loc = ribbon.CreateClone();    
            current_surf(:,kdx) = feval(hCurrentCalc_continuum_forGivenEnergy, DeltaPhi_vec, Evec(kdx), ribbon_loc ); 
        end
        
        
        for jdx = 1:length( DeltaPhi_vec )
            currentvec2integrate =  current_surf(jdx,:);
            % omitting NaNs by substituting interpolates
        	indexes = isnan(currentvec2integrate); 
			indexesall = 1:length(currentvec2integrate);
			        
            currentvec2integrate(indexes) = interp1(indexesall(~indexes), currentvec2integrate(~indexes), indexesall(indexes), 'spline' );
            current_surf(jdx,:) = currentvec2integrate;
			currentvec(jdx) = fact*SimphsonInt( currentvec2integrate.*(deltaEvec) )/(2*pi);            
        end
        
        display('Process finished.', 1)
        
        %-----------------------------------------------------------
        function current = CurrentCalc_continuum_forGivenEnergy( DeltaPhi_vec, E, ribbon_loc )
        
        %if abs(abs(E)-0.6) <= 1e-8
        %    current = zeros(size( DeltaPhi_vec ) );
        %    return
        %end
        WS_ribbon_loc = ribbon_loc.GetWorkspace();
        SetEnergy( E, 'ribbon', ribbon_loc  );  
        
        param = WS_ribbon_loc.FL_handles.Read( 'param' );         
        pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
        
        current = NaN( size( DeltaPhi_vec ) );
        if abs(WS_ribbon.E) <= abs(pair_potential)
    	    return
        end
        
        [current_op,gfininv] = createCurrentOperator('ribbon', ribbon_loc); 
        if isnan(current_op)
            return
        end	 
              
        recalculateSurface = true;
        for idx = 1: length( DeltaPhi_vec )
            % setting phi2 to phi2+delta_phi 
			try   
	            increasePhaseDifference( DeltaPhi_vec(idx)-DeltaPhi_vec(1), ribbon_loc );
 	   	        current_tmp = CurrentCalc_continuum_for_GivenDeltaPhi( ribbon_loc, gfininv, current_op, recalculateSurface) ;
            catch errCause
              	display( 'Error in SNSJosephson:CurrentCalc_continuum_forGivenEnergy caused by:')
               	display( errCause.identifier );
               	display( 'Giving NaN for the calculated current')
               	current_tmp = NaN;
            end
            recalculateSurface = false;
    	    current(idx) = current_tmp;
        end       
        
            
    
        %-------------------------------------------------
        function current = CurrentCalc_continuum_for_GivenDeltaPhi( ribbon_loc, gfininv, current_op, recalculateSurface )
  
            %Dysonfunc = @()CustomDysonFunc('recalculateSurface', recalculateSurface);
            Dysonfunc = @()ribbon_loc.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', false, 'gfininv', gfininv, 'constant_channels', false);
            
            G = WS_ribbon_loc.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
            
            %[S,ny] = WS_ribbon.FL_handles.SmatrixCalc();
            M = WS_ribbon_loc.FL_handles.Read('M');
            
            % incomming states from left (1) and right (2) 
            Surface = WS_ribbon_loc.FL_handles.Read('Surface');
            vcsop_p = cell(length(Surface),1);
            k_p = cell(length(Surface),1);
            k_m = cell(length(Surface),1);
            modusmtx_p = cell(length(Surface),1);
            Normamtx = cell(length(Surface),1);
            modusmtx_m = cell(length(Surface),1);
            d_modusmtx_m = cell(length(Surface),1);
            for iidx = 1:length(Surface)
                vcsop_p{iidx}    = Surface{iidx}.Read('vcsop_p');
                k_p{iidx}        = Surface{iidx}.Read('k_p');
                k_m{iidx}        = Surface{iidx}.Read('k_m');
                modusmtx_p{iidx} = Surface{iidx}.Read('modusmtx_p');
                Normamtx{iidx}   = Surface{iidx}.Read('Normamtx');
                modusmtx_m{iidx} = Surface{iidx}.Read('modusmtx_m');
                d_modusmtx_m{iidx} = Surface{iidx}.Read('d_modusmtx_m');
            end
          
            %propagating indexes
            tolerance = 1e-6;
            prop_indexes{1} = find(abs(imag(k_p{1})) < tolerance);
            prop_indexes{2} = find(abs(imag(k_p{2})) < tolerance);

            current = 0;
            for cell_index = 1:length(Surface)
                for iidx = 1:length(prop_indexes{cell_index})
                    current_tmp = CurrentFromGivenMode( cell_index, prop_indexes{cell_index}(iidx) );
                    current = current + current_tmp;
                end
            end    
       
            if imag(current) > 1e-6
                warning(['Current is complex: ', num2str(current)])
            end
            current = real(current);
                
        
            %--------------------------------------------------
            % calculates current from given incomming mode
            function current_trans = CurrentFromGivenMode( cell_index, idx )
                tolerance = 1e-6;
                modus_in = zeros(size(G,1), 1);
                indexes = sum(M(1:cell_index-1)) + (1:M(cell_index));
                modus_in(indexes) = Normamtx{cell_index}*modusmtx_p{cell_index}(:,idx)/sqrt(vcsop_p{cell_index}(idx));
            
                modus_out = G*modus_in;
                if isempty( WS_ribbon_loc.Surface_interface )
                    % in case no interface region is included
                    sc_wavefnc = modus_out(sum(M)+1:end);        
                else
                    % with interface region included in the setup
                    sc_wavefnc = modus_out(2*sum(M)+1:end);                   
                end
            
            

            
%               particle current
                current_trans = sc_wavefnc'*current_op*sc_wavefnc;
                return

                %quasiparticle current
                quasi_current_trans = 1/(1i)*( c2'*H1sc'*c1 - c1'*H1sc*c2)
            
                % ******** unitarity test **********
                % coupling Hamiltonian to calculate the current in the incomming lead
                H1 = Surface{cell_index}.Read('H1');     

                modus_in = modusmtx_p{cell_index}(:,idx)/sqrt(vcsop_p{cell_index}(idx));
                quasi_current_in = 1i*modus_in'*(H1*exp(1i*k_p{cell_index}(idx))-H1'*exp(-1i*k_p{cell_index}(idx)))*modus_in;
            
                modus_ref = modus_out(indexes) - modus_in;
                r = d_modusmtx_m{cell_index}*modus_ref;
                quasi_current_ref = 0;
                for iiidx = 1:M(cell_index)
                    if abs(imag(k_m{cell_index}(iiidx))) <= 1e-6
                        modus_ref_tmp = r(iiidx)*modusmtx_m{cell_index}(:,iiidx);
                        quasi_current_ref = quasi_current_ref + 1i*modus_ref_tmp'*(H1*exp(1i*k_m{cell_index}(iiidx))-H1'*exp(-1i*k_m{cell_index}(iiidx)))*modus_ref_tmp;
                    end
                end
                quasi_current_ref = -quasi_current_ref
            
                delta_quasi_current = quasi_current_in - quasi_current_trans - quasi_current_ref;
            	delta_quasi_current
                if abs(delta_quasi_current) > tolerance
                    display(['unitarity test failed with \Delta I = ',num2str(delta_quasi_current)]);
                end 
                % ******** unitarity test **********
            
            end
        
        end
    
        end
    
    end

%% createCurrentOperator
% Calculates the charge current operator from the inverse of the Greens function of the scattering region.
	function [current_op, gfininv] = createCurrentOperator( varargin )
        
        p = inputParser;
        p.addParamValue('ribbon', ribbon);
        p.parse(varargin{:});
        
        ribbon_loc = p.Results.ribbon;
        WS_ribbon_loc = ribbon_loc.GetWorkspace();
        
		fE = Fermi(WS_ribbon_loc.E);
		
		gfininv = create_scatter_GreensFunction('onlyAtEnds', true, 'ribbon', ribbon_loc);
		if isnan(gfininv)
			current_op = NaN;
			return
		end
		
		M = size(gfininv)/4;
	
		current_op = zeros( size(gfininv) );
		% gfininv = E - Heff(E) !!!!!
		current_op( 1:M, 2*M+1:3*M ) = -fE/1i*(-1)*gfininv( 1:M, 2*M+1:3*M );
		current_op( M+1:2*M, 3*M+1:end ) = -(1-fE)/1i*(-1)*gfininv( M+1:2*M, 3*M+1:end );
		current_op( 2*M+1:3*M, 1:M ) = fE/1i*(-1)*gfininv( 2*M+1:3*M, 1:M );
		current_op( 3*M+1:end, M+1:2*M ) = (1-fE)/1i*(-1)*gfininv(3*M+1:end, M+1:2*M );
	
    end


%% getBandWidth
% Determines the band width in the leads and in the scattering region.
    function getBandWidth()
    	display('Getting Bandwidths', 1);
        
        if ~isempty(BandWidth)
            return
        end
        
        Surface_1 = WS_ribbon.FL_handles.SurfaceGreenFunctionCalculator(1, 'Just_Create_Hamiltonians', true);
        W = BandWidthCalculator( Surface_1 );
        
        param = WS_ribbon.FL_handles.Read( 'param' );         
        pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
        
        BandWidth.Lead = struct( 'Emin', pair_potential, 'Emax', W);
        
                
        ribbon.CreateRibbon('justHamiltonians', true);        
        W = BandWidthCalculator( WS_ribbon.Surface_tmp );
        BandWidth.Scatter = struct( 'Emin', 0, 'Emax', W);
         
         %--------------------------------------------
         function W = BandWidthCalculator( Surface_tmp )
         
         	H0 = Surface_tmp.Read('H0');
        	H1 = Surface_tmp.Read('H1'); 
        	u = rand(size(H0,1)); 
        	u = u/norm(u);     

            if exist('parpool' )
            Opt = WS_ribbon.FL_handles.Read('Opt');
            	if ~isempty(Opt.workers)
                	poolobj = parpool(Opt.workers);
            	else
                	poolobj = parpool();
            	end
        	else
            	poolobj = [];
        	end
            
            hgetMaxEigenvalue = @getMaxEigenvalue;
            hsecular_H = @secular_H;
            k = -pi:2*pi/100:pi;
            arnoldi = zeros( length(k),1);
            parfor idx = 1:length(k)
                Heff = feval(hsecular_H,H0,H1,k(idx));
                arnoldi(idx) = abs(feval(hgetMaxEigenvalue, Heff ));                
            end
            W = max( arnoldi );
            
            if ~isempty(poolobj)
                delete(poolobj)
            end

         	% --------------------------------------
         	% Hamiltonian for the secular equation
         	function H = secular_H(H0,H1,k)     
         		H = H0 + H1*exp(1i*k) + H1'*exp(-1i*k);    
         	end
         
            %--------------------------------------------
         	% getting the maximal eigenvalue with Arnoldi iteration
         	% Numerical Linear Algebra" by Trefethen and Bau. (p250) or
         	% http://en.wikipedia.org/wiki/Arnoldi_iteration
         	function ret = getMaxEigenvalue( Heff )
         		%u = rand(size(H0,1)); 
        		%u = u/norm(u);
         		maxIter = 1e2;
         		tolerance = 1e-5;
         	    ret = 0;
         		lambda = 1;
         		for iidx=1:maxIter
         			u_new = Heff*u;
         			norm_new = norm(u_new);
         			lambda_new = norm_new/norm(u);
         		
         			if abs((lambda - lambda_new)/lambda) < tolerance
         				ret = lambda;
         				return
         			end         		
         			u = u_new/norm_new;
         			lambda = lambda_new;     	
         		end
         	end
         end
        
    end


%% CreateHandlesForMagneticField
% Creates the function handles of the vector potentials and apply the magnetic filed in the ribbon Hamiltonians.
    function CreateHandlesForMagneticField()  
        [hLandaux, hLandauy, gauge_field ] = createVectorPotentials( WS.B );
        ribbon.setHandlesForMagneticField('scatter', hLandauy, 'lead', hLandaux, 'gauge_field', gauge_field );
    end


%% create_scatter_GreensFunction
% Calculates the surface Greens function of the scattering region.
    function gfininv = create_scatter_GreensFunction(  varargin )
        
        p = inputParser;
        p.addParamValue('onlyAtEnds', false ); 
        p.addParamValue('gauge_trans', true); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians
        p.addParamValue('ribbon', ribbon); %for parallel computations
        p.parse(varargin{:});       
        onlyAtEnds     = p.Results.onlyAtEnds;
        gauge_trans     = p.Results.gauge_trans;
        ribbon_loc     = p.Results.ribbon;
        
        WS_ribbon_loc = ribbon_loc.GetWorkspace();
        
        try
        	ribbon_loc.CreateRibbon( 'ContactPotInterface', ContactPotInterface, 'forcedInterface', true);    
		catch errCause
            display( 'Error in SNSJosephson:create_scatter_GreensFunction caused by:')
            display( errCause.identifier );
            display( 'Giving NaN for the Greens function')
            gfin = NaN;
            gfininv = NaN;
            return
        end 
        
        coordinates_tmp = WS_ribbon_loc.Surface_tmp.Read( 'coordinates' );
        coordinates_scatter = structures('coordinates');
        
        z1 = 0;
        z2 = WS_ribbon_loc.height;
        z3 = round( WS_ribbon_loc.height/2 )+1;
        z3_coords = [z3-1, z3];%, z3+1];
        %z3_coords = [z3];
        
        coordinates_scatter.x = [coordinates_tmp.x+coordinates_tmp.a(1)*z1; coordinates_tmp.x+coordinates_tmp.a(1)*z2];
        coordinates_scatter.y = [coordinates_tmp.y+coordinates_tmp.a(2)*z1; coordinates_tmp.y+coordinates_tmp.a(2)*z2];
        if ~isempty( coordinates_tmp.BdG_u )
            coordinates_scatter.BdG_u = [coordinates_tmp.BdG_u; coordinates_tmp.BdG_u];
        end
        
        
        M = WS_ribbon_loc.Surface_tmp.Read('M');
        
        if onlyAtEnds
            WS_ribbon_loc.Surface_tmp.FiniteGreenFunction(z1,z2, 'onlygfininv', true);
            gfininv = WS_ribbon_loc.Surface_tmp.Read('gfininv');
        else
            gfin = GreensFuncAtRibbonEnds_z1z2z3();
            if isnan(rcond(gfin) ) || abs( rcond(gfin) ) < 1e-16
                gfininv = CommonFunctionsHandle.inv_SVD( gfin );
            else
                gfininv = inv(gfin);
            end       
            indexes = [1+M:3*M,4*M+1:(4+length(z3_coords))*M];
            gfininv = gfininv( indexes, indexes );
            for kdx = 1:length( z3_coords )
                coordinates_scatter.x = [coordinates_scatter.x; coordinates_tmp.x+coordinates_tmp.a(1)*z3_coords(kdx) ];
                coordinates_scatter.y = [coordinates_scatter.y; coordinates_tmp.x+coordinates_tmp.a(2)*z3_coords(kdx) ];
                if isfield( coordinates_tmp, 'BdG_u' )
                    coordinates_scatter.BdG_u = [coordinates_scatter.BdG_u; coordinates_tmp.BdG_u];
                end
            end
        end
       
        
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans && ~isempty(  WS_ribbon_loc.PeierlsTransform_Scatter )
            gfininv = WS_ribbon_loc.PeierlsTransform_Scatter.gaugeTransformation( gfininv, coordinates_scatter, WS_ribbon_loc.gauge_field );
            WS_ribbon_loc.PeierlsTransform_Scatter.gaugeTransformationOnLead( WS_ribbon_loc.Surface_tmp, WS_ribbon_loc.gauge_field );  
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2z3()
            
            gfin = zeros(4*M+length(z3_coords)*M, 4*M+length(z3_coords)*M);
            gfin(1:4*M,1:4*M) = GreensFuncAtRibbonEnds_z1z2();
            
            
            for idx = 1:length(z3_coords) %oszlop index
                z3_indexes = 4*M+(idx-1)*M + (1:M);
                
                for jdx = 1:length(z3_coords) %sor index
                    
                    if idx == jdx
                        gtmp = gfin(1:M,1:M);
                        gfin( z3_indexes, z3_indexes) = gtmp;
                    elseif z3_coords(idx)-1==z3_coords(jdx)
                        gtmp = gfin(1:M,1+M:2*M);
                        gfin( z3_indexes-M, z3_indexes) = gtmp;                        
                    elseif z3_coords(idx)+1==z3_coords(jdx)
                        gtmp = gfin(1+M:2*M,1:M);
                        gfin( z3_indexes+M, z3_indexes) = gtmp;   
                    else
                        z3_indexes2 = 4*M+(jdx-1)*M + (1:M);
                        gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(jdx), z3_coords(idx));
                        gfin(z3_indexes2, z3_indexes)     = gtmp;
                    end                
                end
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z3_coords(idx));
                gfin(1:M, z3_indexes)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1-1);
                gfin(z3_indexes, 1:M)     = gtmp;    
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1, z3_coords(idx));
                gfin(1+M:2*M, z3_indexes)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1);
                gfin(z3_indexes, 1+M:2*M)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2, z3_coords(idx));
                gfin(1+2*M:3*M, z3_indexes)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2);
                gfin(z3_indexes, 1+2*M:3*M)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z3_coords(idx));
                gfin(1+3*M:4*M, z3_indexes)     = gtmp;
                
                gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2+1);
                gfin(z3_indexes, 1+3*M:4*M)     = gtmp; 
                
                
            end
            
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2()
            gfin = zeros(4*M,4*M);
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z1-1);
            gfin(1:M,1:M)              = gtmp;
            gfin(1+M:2*M, 1+M:2*M)     = gtmp;
            gfin(1+2*M:3*M, 1+2*M:3*M) = gtmp;              
            gfin(1+3*M:4*M, 1+3*M:4*M) = gtmp;
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z1);
            gfin(1:M,1+M:2*M) = gtmp;
            gfin(1+2*M:3*M,1+3*M:4*M) = gtmp;
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1, z1-1);
            gfin(1+M:2*M,1:M) = gtmp;
            gfin(1+3*M:4*M, 1+2*M:3*M) = gtmp;    
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z2);
            gfin(1:M,1+2*M:3*M) = gtmp; 
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2, z1-1);
            gfin(1+2*M:3*M,1:M) = gtmp; 
                
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z2+1);
            gfin(1:M,1+3*M:4*M) = gtmp; 
                
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z1-1);
            gfin(1+3*M:4*M,1:M) = gtmp;   
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1, z2);
            gfin(1+M:2*M,1+2*M:3*M) = gtmp; 
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2, z1);
            gfin(1+2*M:3*M,1+M:2*M) = gtmp; 
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z1, z2+1);
            gfin(1+M:2*M,1+3*M:4*M) = gtmp; 
        
            gtmp = WS_ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z1);
            gfin(1+3*M:4*M,1+M:2*M) = gtmp;  
            
        end     
        
    end


%% GetWorkspace()%
% Return the handle class to the workspace of the interface
    function ret = GetWorkspace()
        ret = WS;        
    end


% This increase phase difference between the Leads by delta_phi
    function increasePhaseDifference( delta_phi, ribbon_loc )
        
        WS_ribbon_loc = ribbon_loc.GetWorkspace(); %in order to use custom ribbon, for parallel computations
        
        param = WS_ribbon_loc.FL_handles.Read( 'param' ); 
        Surface = WS_ribbon_loc.FL_handles.Read( 'Surface' ); 
        
        if isempty( Surface )
            return
        end
                        
        param.Leads{2}.pair_potential = param.Leads{2}.pair_potential*exp(1i*delta_phi);
        
        Surface{2}.Write( 'param', param);  %reset is invoked automatically when rewriting param
        WS_ribbon_loc.FL_handles.SurfaceGreenFunctionCalculator(2, 'Surface_tmp', Surface{2}, 'SelfEnergy', useSelfEnergy, 'SurfaceGreensFunction', ~useSelfEnergy, ...
            'gauge_field', WS_ribbon_loc.gauge_field);
        
        
        % recalculating the interface region
        if isempty( WS_ribbon_loc.Surface_interface )
            return
        end
        
        display('Creating interface Hamiltonian with the new pair potential')
        Surface_interface = WS_ribbon_loc.Surface_interface{2};
        Surface_interface.Write( 'param', param);  %reset is invoked automatically when rewriting param
        ribbon_undecimated.setInterfaceRegions( WS_ribbon_loc.Surface_interface );
        ribbon_undecimated.CreateInterface( 2, 'ContactPotInterface', ContactPotInterface );
        ribbon_undecimated.DecimateInterface( 2, 'E', WS_ribbon_loc.E );
        
        
    end

% Fermi distribution
    function ret = Fermi(E)
        if abs(WS.T) <= 1e-6
    		if E < 0
    			ret = 1;
    		elseif E == 0
    			ret = 0.5;
    		else
    			ret = 0;
    		end
    	else
        	ret = 1/(1+exp((E)*beta));
        end
    end   




% creates function handles for the vector potential and gauge tarnsformation
    function [hLandaux, hLandauy, hgauge_field ] = createVectorPotentials( B )
        
        rCC = WS_ribbon.rCC*1e-10; %Angstrom
		phi0 = WS_ribbon.h/WS_ribbon.qe;
        eta_B = 2*pi/phi0*(rCC)^2*B;
        Aconst = 0.1; % constant vektor potential in the systems: stabilizes the numerical computation
        
        Opt = WS_ribbon.FL_handles.Read( 'Opt' );
        
        if strcmp(Opt.Lattice_Type, 'Square')
            lattice_constant = 1;
        else
            lattice_constant = sqrt(3);
        end
        
        %Opt.gauge = 'LandauY';
        %Opt.eta = eta_B;
        
        hLandaux = @Landaux;
        %hLandauy = @(x,y)([zeros( 1,length(x) );0.5*ones( 1,length(x) )]);
        hLandauy = @(x,y)([0;eta_B*x]);
        hgauge_field = @gauge_field;
        
        
        %---------------------------------
        % gauge scalar field for transformation from (i) Landauy to Landaux for 0<y<height and (ii) from zero field to constant [0, Ay]
        function ret = gauge_field(x,y)
            %ret = -0.5*y - 0.5*x;
            %return
            ret = zeros( length(x),1 );
            
            indexes1 = logical( y<=0 );
            ret( indexes1 ) = -Aconst*x(indexes1);
            
            indexes2 = logical( y>=WS_ribbon.height*lattice_constant );
            ret( indexes2 ) = -eta_B*WS_ribbon.height*lattice_constant*x(indexes2) - Aconst*x(indexes2);
            
            indexes3 = ~( indexes1 | indexes2 );
            ret( indexes3 ) = -eta_B*x(indexes3).*y(indexes3) - Aconst*x(indexes3);  
        end
              
        
        %-----------------------------------
        function ret = Landaux( x,y )
            %ret = -[0.5*ones( 1,length(x) ); zeros( 1,length(x) )];
            %return
            ret = zeros( length(x),2 );
            
            indexes1 = logical( y<=0 );
            ret( indexes1, 1 ) = -Aconst;
            
            indexes2 = logical( y>=WS_ribbon.height*lattice_constant );
            ret( indexes2, 1 ) = -eta_B*WS_ribbon.height*lattice_constant-Aconst;
            
            indexes3 = ~( indexes1 | indexes2 );
            ret( indexes3, 1) = -eta_B*y(indexes3)-Aconst;
        end
        
    end


% Simplhson integral: y is a vector of function values at equal distant points: y_i = f(x_i), x_i-x_{i-1}=h
    function ret = SimphsonInt(y,h)
        if ~exist( 'h', 'var' )
            h = 1;
        end
        
        if mod(length(y),2) == 0
            warning('length of the vector to integral has to be (2n+1)')
            y(end) = [];
        end
        
        ya = y(1);
        yb = y(end);
        y(1) = [];
        
        ret = h/3*( ya + yb ) + 2*h/3*sum(y(2:2:end-2)) + 4*h/3*sum(y(1:2:end-1));
        
        
    end






% sets the energy
    function SetEnergy( newE, varargin ) 
        p = inputParser;
        p.addParamValue('ribbon', ribbon);
        p.parse(varargin{:});
        
        ribbon_loc = p.Results.ribbon;
        ribbon_loc.setEnergy( newE ) 
    end

% displays output messages on the screen
    function display( message, nosilent )
        if ~WS_ribbon.silent || exist('nosilent', 'var')
            disp( message )
        end
       
        Opt = WS_ribbon.FL_handles.Read( 'Opt' );
        if Opt.debug
            fid = fopen('debug.txt', 'a');
            fprintf(fid, [message,'\n']);
            fclose(fid);
        end
        
    end

% Input parsing
    function InputParsing()
        
        p = inputParser;
        p.addParamValue('width', 1, @isscalar);
        p.addParamValue('height', 8, @isscalar);
        p.addParamValue('E', 0.637, @isscalar);
        p.addParamValue('EF', 0, @isscalar);
        p.addParamValue('phi', [[],[]]);
        p.addParamValue('T', WS.T, @isscalar);
        p.addParamValue('B', WS.B, @isscalar);
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_zigzag_leads.xml'], @ischar);
        p.addParamValue('silent', 0);
        p.addParamValue('ContactPotInterface', []);
        
        p.parse(varargin{:});

       
        width     = p.Results.width;
        height    = p.Results.height;
        E            = p.Results.E;
        EF           = p.Results.EF;
        phi          = p.Results.phi;
        WS.T            = p.Results.T;
        WS.B         = p.Results.B;
        silent       = p.Results.silent;
        filenameIn       = p.Results.filenameIn;
        filenameOut       = p.Results.filenameOut;
        ContactPotInterface = p.Results.ContactPotInterface;
        
        if mod(width,2) == 1 && width ~= 1
            display('correcting width to be even')
            width = width + 1;
        end
        
        
        beta = 1/(k_B*WS.T);      
        ribbon = Ribbon('width', width, 'height', height, 'E', E, 'EF', EF, 'phi', phi, 'silent', silent, 'filenameIn', filenameIn, 'filenameOut', filenameOut );
        WS_ribbon = ribbon.GetWorkspace();

		Opt = WS_ribbon.FL_handles.Read('Opt');
		if Opt.magnetic_field
			CreateHandlesForMagneticField( )
		end
        
        WS_ribbon.FL_handles.LeadCalc('createCore', true );
        ribbon_undecimated = ribbon.CreateClone();
               

    end
end
